@@ -11,6 +11,12 @@ struct deg_list {
 	int node;
 };
 
-int *min_deg(int,int *,int *,int);
+struct pair_wt {
+	int pair_node;
+	double wt;
+};
+
+int *min_deg(int,int *,int *,int *,int);
+int *greedy(int,int *,int *,int *,double *,struct pair_wt *,int,double *);
 
 #endif
