@@ -37,40 +37,43 @@ static struct gp_rfunc *free_rf_list[MAX_GP_RF_GENES],**autozyg_list;
 static struct deg_list *deglist,**first;
 static int *gp_inv,*gp_inv1,*gp_flag,*pflag,*rf_idx,
   **fact,*ofact,rf_idx_n,*rfxn,**rfx,
-  *cnv_gene,gp_xx_size;
+  *cnv_gene,gp_xx_size,mem_list_size,mem_list_ptr;
 static double *gp_tmp_p,*gp_z,*gp_xx;
+static void **mem_list;
+
+
+static void *realloc_and_remember(void *p,size_t size)
+{
+	int i;
+	
+	for(i=0;i<mem_list_ptr;i++) if(mem_list[i]==p) break;
+	if(i<mem_list_ptr) {
+		p=realloc(p,size);
+		mem_list[i]=p;
+	} else p=0;
+	return p;
+}
 
 #ifdef FUNC_NAME
 #undef FUNC_NAME
 #endif
-#define FUNC_NAME "alloc_gen_pen"
-void alloc_gen_pen(int ngenes)
+#define FUNC_NAME "malloc_and_remember"
+static void *malloc_and_remember(size_t size)
 {
-	int i,n_all;
+	void *p;
 	
-#ifdef TRACE_PEEL
-	if(CHK_PEEL(TRACE_LEVEL_1)) (void)printf("In alloc_gen_pen(%d)\n",ngenes);
-#endif
-	if(!(rflist=malloc(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
-	if(!(autozyg_list=malloc(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
-	if(!(rfuncs=malloc(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
-	if(!(first=malloc(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
-	if(!(fact=malloc(sizeof(void *)*2*ngenes))) ABT_FUNC(MMsg);
-	rfx=fact+ngenes;
-	if(!(deglist=malloc(sizeof(struct deg_list)*ngenes))) ABT_FUNC(MMsg);
-	for(i=0;i<ngenes;i++) deglist[i].gene=i;
-	if(!(gp_inv=malloc(sizeof(int)*7*ngenes))) ABT_FUNC(MMsg);
-	gp_inv1=gp_inv+ngenes;
-	gp_flag=gp_inv1+ngenes;
-	pflag=gp_flag+ngenes;
-	ofact=pflag+ngenes;
-	rfxn=ofact+ngenes;
-	cnv_gene=rfxn+ngenes;
-	n_all=2;
-	if(!(gp_tmp_p=malloc(sizeof(double)*(ngenes+n_all*n_all)))) ABT_FUNC(MMsg);
-	gp_z=gp_tmp_p+n_all*n_all;
-	gp_xx_size=8;
-	if(!(gp_xx=malloc(sizeof(double)*gp_xx_size))) ABT_FUNC(MMsg);
+	if(mem_list_ptr==mem_list_size) {
+		if(mem_list_size) {
+			mem_list_size*=1.5;
+			if(!(mem_list=realloc(mem_list,sizeof(void *)*mem_list_size))) ABT_FUNC(MMsg);
+		} else {
+			mem_list_size=16;
+			if(!(mem_list=malloc(sizeof(void *)*mem_list_size))) ABT_FUNC(MMsg);
+		}
+	}
+	p=malloc(size);
+	if(p) mem_list[mem_list_ptr++]=p;
+	return p;
 }
 
 static void free_rfunc(struct gp_rfunc *rf)
@@ -86,10 +89,6 @@ static void free_rfunc(struct gp_rfunc *rf)
 void free_gen_pen(void) 
 {
 	int i=0;
-	void **memp[]={(void **)&rflist,(void **)&deglist,(void **)&first,
-		  (void **)&gp_inv,(void **)&rfuncs,(void **)&gp_tmp_p,
-		  (void **)&rf_idx,(void **)&fact,(void **)&autozyg_list,
-		  (void **)&gp_xx,0},**p;
 	struct gp_rfunc_ptr *rfp,*rfp1;
 	struct gp_rfnode *rfn,*rfn1;
 	struct gp_rfunc *rf,*rf1;
@@ -112,12 +111,11 @@ void free_gen_pen(void)
 		rfn=rfn1;
 	}
 	free_rfn_list=0;
-	while(memp[i]) {
-		p=memp[i++];
-		if(*p) {
-			free(*p);
-			*p=0;
-		}
+	if(mem_list) {
+		for(i=0;i<mem_list_ptr;i++) if(mem_list[i]) free(mem_list[i]);
+		free(mem_list);
+		mem_list=0;
+		mem_list_size=mem_list_ptr=0;
 	}
 	rf_idx_n=0;
 	for(i=0;i<MAX_GP_RF_GENES;i++) {
@@ -139,6 +137,39 @@ void free_gen_pen(void)
 	}
 }
 
+#ifdef FUNC_NAME
+#undef FUNC_NAME
+#endif
+#define FUNC_NAME "alloc_gen_pen"
+void alloc_gen_pen(int ngenes)
+{
+	int i,n_all;
+	
+#ifdef TRACE_PEEL
+	if(CHK_PEEL(TRACE_LEVEL_1)) (void)printf("In alloc_gen_pen(%d)\n",ngenes);
+#endif
+	if(!(rflist=malloc_and_remember(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
+	if(!(autozyg_list=malloc_and_remember(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
+	if(!(rfuncs=malloc_and_remember(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
+	if(!(first=malloc_and_remember(sizeof(void *)*ngenes))) ABT_FUNC(MMsg);
+	if(!(fact=malloc_and_remember(sizeof(void *)*2*ngenes))) ABT_FUNC(MMsg);
+	rfx=fact+ngenes;
+	if(!(deglist=malloc_and_remember(sizeof(struct deg_list)*ngenes))) ABT_FUNC(MMsg);
+	for(i=0;i<ngenes;i++) deglist[i].gene=i;
+	if(!(gp_inv=malloc_and_remember(sizeof(int)*7*ngenes))) ABT_FUNC(MMsg);
+	gp_inv1=gp_inv+ngenes;
+	gp_flag=gp_inv1+ngenes;
+	pflag=gp_flag+ngenes;
+	ofact=pflag+ngenes;
+	rfxn=ofact+ngenes;
+	cnv_gene=rfxn+ngenes;
+	n_all=2;
+	if(!(gp_tmp_p=malloc_and_remember(sizeof(double)*(ngenes+n_all*n_all)))) ABT_FUNC(MMsg);
+	gp_z=gp_tmp_p+n_all*n_all;
+	gp_xx_size=8;
+	if(!(gp_xx=malloc_and_remember(sizeof(double)*gp_xx_size))) ABT_FUNC(MMsg);
+}
+
 #ifdef FUNC_NAME
 #undef FUNC_NAME
 #endif
@@ -201,7 +232,9 @@ static struct gp_rfnode *find_rfnode(int x,int y)
 	}
 	if(!p || p->y!=y) {
 		if((p=free_rfn_list)) free_rfn_list=p->next;
-		else if(!(p=malloc(sizeof(struct gp_rfnode)))) ABT_FUNC(MMsg);
+		else {
+			if(!(p=malloc(sizeof(struct gp_rfnode)))) ABT_FUNC(MMsg);
+		}
 		p->next=*pp;
 		p->y=y;
 		p->rf=0;
@@ -863,7 +896,36 @@ double gen_pen(int loc,int c,int *err,int flag,int si)
 					rfp1=rfp2;
 				}
 				rfuncs[k]=0;
-			} else pflag[i]=0;
+			} else {
+				pflag[i]=0;
+				rfp3=rfuncs+k;
+				rfp1=*rfp3;
+				while(rfp1) {
+					rf1=rfp1->rf;
+					if(!rf1->flag) {
+						for(k1=0;k1<rf1->n_inv;k1++) {
+							k3=rf1->inv[k1];
+							for(k2=0;k2<n_inv;k2++) if(gp_inv1[k2]==k3) break;
+							if(k2==n_inv) break;
+						}
+						if(k1==rf1->n_inv) {
+							rf1->next=rfl;
+							rfl=rf1;
+							rf1->flag=1;
+							n_rf++;
+						}
+					}
+					if(rf1->flag) {
+#ifdef TRACE_PEEL
+						if(CHK_PEEL(TRACE_LEVEL_3)) (void)printf("\n[%s:%d] %s() - Adding %p to free_rfp_list\n",__FILE__,__LINE__,FUNC_NAME,(void *)rfp1);
+#endif
+						*rfp3=rfp1->next;
+						rfp1->next=free_rfp_list;
+						free_rfp_list=rfp1;
+					} else rfp3=&rfp1->next;
+					rfp1=*rfp3;
+				}
+			}
 		}
 #ifdef TRACE_PEEL
 		if(CHK_PEEL(TRACE_LEVEL_2)) {
@@ -934,8 +996,8 @@ double gen_pen(int loc,int c,int *err,int flag,int si)
 		/* Check we have enough space for rfunction indices */
 		if(n_rf>rf_idx_n) {
 			rf_idx_n=n_rf;
-			if(rf_idx) rf_idx=realloc(rf_idx,sizeof(int)*(rf_idx_n+2*rf_idx_n*ngenes));
-			else rf_idx=malloc(sizeof(int)*(rf_idx_n+2*rf_idx_n*ngenes));
+			if(rf_idx) rf_idx=realloc_and_remember(rf_idx,sizeof(int)*(rf_idx_n+2*rf_idx_n*ngenes));
+			else rf_idx=malloc_and_remember(sizeof(int)*(rf_idx_n+2*rf_idx_n*ngenes));
 			if(!rf_idx) ABT_FUNC(MMsg);
 			fact[0]=rf_idx+rf_idx_n;
 			rfx[0]=fact[0]+rf_idx_n*ngenes;
@@ -948,12 +1010,19 @@ double gen_pen(int loc,int c,int *err,int flag,int si)
 		if(!flag || n_out) {
 			/* Perform peelop */
 			/* Initialize frequency probs. and allele indices */
-			if(n_genetic_groups>1) z1=freq[grp[0]][0];
-			else z1=freq1[0];
-			for(z=1.0,i=n_inv-1;i>=0;i--) {
-				gp_inv[i]=0;
-				gp_z[i]=z;
-				if(pflag[i]) z*=z1;
+			if(n_genetic_groups>1) {
+				for(z=1.0,i=n_inv-1;i>=0;i--) {
+					gp_inv[i]=0;
+					gp_z[i]=z;
+					if(pflag[i]) z*=freq[grp[gp_inv1[i]]][0]; 
+				}
+			} else {
+				z1=freq1[0];
+				for(z=1.0,i=n_inv-1;i>=0;i--) {
+					gp_inv[i]=0;
+					gp_z[i]=z;
+					if(pflag[i]) z*=z1;
+				}
 			}
 			z3=z;
 			/* Initialize output index */
@@ -1251,26 +1320,44 @@ double gen_pen(int loc,int c,int *err,int flag,int si)
 			/* Set up temporary storage */
 			nc=(int)(.5+exp(log((double)n_all)*(double)n_peel));
 			if(nc>gp_xx_size) {
-				if(gp_xx) free(gp_xx);
-				if(!(gp_xx=malloc(sizeof(double)*nc))) ABT_FUNC(MMsg);
+				if(gp_xx) {
+					if(!(gp_xx=realloc_and_remember(gp_xx,sizeof(double)*nc))) ABT_FUNC(MMsg);
+				} else {
+					if(!(gp_xx=malloc_and_remember(sizeof(double)*nc))) ABT_FUNC(MMsg);
+				}
 				gp_xx_size=nc;
 			}
 			xx=gp_xx;
 			for(i=0;i<nc;i++) xx[i]=0.0;
 			/* Set allele indices and frequency probs. */
-			if(n_genetic_groups>1) z1=freq[grp[0]][0];
-			else z1=freq1[0];
-			for(z=1.0,i=n_inv-1;i>=0;i--) {
-				gp_z[i]=z;
-				if(pflag1[i]) {
-					gp_inv[i]=0;
-					z*=z1;
-				} else {
-					k=gp_inv1[inv1[i]]-1;
+			if(n_genetic_groups>1) {
+				for(z=1.0,i=n_inv-1;i>=0;i--) {
+					gp_z[i]=z;
+					if(pflag1[i]) {
+						gp_inv[i]=0;
+						z*=freq[grp[inv1[i]]][0];
+					} else {
+						k=gp_inv1[inv1[i]]-1;
 #ifdef DEBUG
-					if(k<0 || k>=n_all) ABT_FUNC("Internal error - unsampled allele\n");
+						if(k<0 || k>=n_all) ABT_FUNC("Internal error - unsampled allele\n");
 #endif
-					gp_inv[i]=k;
+						gp_inv[i]=k;
+					}
+				}
+			} else {
+				z1=freq1[0];
+				for(z=1.0,i=n_inv-1;i>=0;i--) {
+					gp_z[i]=z;
+					if(pflag1[i]) {
+						gp_inv[i]=0;
+						z*=z1;
+					} else {
+						k=gp_inv1[inv1[i]]-1;
+#ifdef DEBUG
+						if(k<0 || k>=n_all) ABT_FUNC("Internal error - unsampled allele\n");
+#endif
+						gp_inv[i]=k;
+					}
 				}
 			}
 			/* Initialize output index */
