@@ -51,7 +51,7 @@ static void get_gts(lk_ulong x,const int n,int *gt)
 	int i=0;
 	lk_ulong a;
 	
-	a=(1<<(n_bits1))-1;
+	a=(LK_ONE<<(n_bits1))-LK_ONE;
 	while(x)	{
 		gt[i++]=1+(int)(x&a);
 		x>>=n_bits1;
@@ -263,6 +263,7 @@ static void free_hash_blocks(void)
 	hash_block=first_hash_block;
 	while(hash_block) {
 		if(hash_block->elements) free(hash_block->elements);
+		if(hash_block->hd) free(hash_block->hd);
 		hb1=hash_block->next;
 		free(hash_block);
 		hash_block=hb1;
@@ -485,13 +486,13 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 			n_ind=rf[j].n_ind;
 			a=0;
 			k2=0;
-			b=(1<<n_bits1)-1;
+			b=(LK_ONE<<n_bits1)-LK_ONE;
 			for(k=n_ind-1;k>=0;k--) {
 				a<<=n_bits1;
 				k1=rf[j].id_list[k];
 				if(gt_store[k1]) a|=b;
 				else {
-					k2|=1<<k1;
+					k2|=LK_ONE<<k1;
 					gt_store[k1]=1;
 				}
 			}
@@ -547,7 +548,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 				}
 				complex_freq[n_fnd][k2]=0.0;
 				for(k1=0;k1<n_all;k1++) {
-					if(a&(1<<k1)) complex_freq[n_fnd][k2]+=freq[k3][k1];
+					if(a&(LK_ONE<<k1)) complex_freq[n_fnd][k2]+=freq[k3][k1];
 					else complex_freq[n_fnd][k1]=freq[k3][k1];
 				}
 				fnd_list[n_fnd++]=k;
@@ -677,7 +678,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 						k1=jnt_idx[k];
 						k2=jnt_list[k1];
 						j=abs(inv[k1])-1;
-						if(!(a_set[j][gt_store[k1]-1]&(1<<(gt_store[k2]-1)))) {
+						if(!(a_set[j][gt_store[k1]-1]&(LK_ONE<<(gt_store[k2]-1)))) {
 							ef=1;
 							break;
 						}
@@ -690,14 +691,14 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 						if(j<0) {
 							j=-j-1;
 							tp=id_array[j].tpp[X_PAT];
-							m=1<<(gt_store[k1]-1);
+							m=LK_ONE<<(gt_store[k1]-1);
 							a=req_set[X_PAT][locus][j];
 							if(a&m) m=a;
 							z=0.0;
 							all=gt_store[off_index[X_MAT][k]];
-							if(m&(1<<(all-1))) z=tp[X_MAT];
+							if(m&(LK_ONE<<(all-1))) z=tp[X_MAT];
 							all=gt_store[off_index[X_PAT][k]];
-							if(m&(1<<(all-1))) z+=tp[X_PAT];
+							if(m&(LK_ONE<<(all-1))) z+=tp[X_PAT];
 							p1*=z;
 							if(p1==0.0)	{
 								ef=1;
@@ -706,14 +707,14 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 						} else {
 							j--;
 							tp=id_array[j].tpp[X_MAT];
-							m=1<<(gt_store[k1]-1);
+							m=LK_ONE<<(gt_store[k1]-1);
 							a=req_set[X_MAT][locus][j];
 							if(a&m) m=a;
 							z=0.0;
 							all=gt_store[off_index[X_MAT][k]];
-							if(m&(1<<(all-1))) z=tp[X_MAT];
+							if(m&(LK_ONE<<(all-1))) z=tp[X_MAT];
 							all=gt_store[off_index[X_PAT][k]];
-							if(m&(1<<(all-1))) z+=tp[X_PAT];
+							if(m&(LK_ONE<<(all-1))) z+=tp[X_PAT];
 							p1*=z;
 							if(p1==0.0)	{
 								ef=1;
@@ -806,7 +807,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 				k2=jnt_list[k1];
 				if(inv[k1]>0) {
 					if(gt_store[k2]) {
-						b=1<<(gt_store[k2]-1);
+						b=LK_ONE<<(gt_store[k2]-1);
 						for(k=other_ptr[i];k<n_all;k++) if(a_set[j][k]&b) {
 							gt_store[k1]=k+1;
 							break;
@@ -835,7 +836,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 							if((other_ptr[i]=k3)==n_all) ef=1;
 						} else {
 							for(k4=other_ptr[k];k4<n_all;k4++) {
-								a=1<<k4;
+								a=LK_ONE<<k4;
 								if(id_array[j].temp[X_PAT]&a)	{
 									for(k3=other_ptr[i];k3<n_all;k3++) if(a_set[j][k3]&a) {
 										gt_store[k1]=k3+1;
@@ -855,7 +856,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 				} else {
 					if(gt_store[k2]) {
 						b=a_set[j][gt_store[k2]-1];
-						for(k=other_ptr[i];k<n_all;k++) if(b&(1<<k))	{
+						for(k=other_ptr[i];k<n_all;k++) if(b&(LK_ONE<<k))	{
 							gt_store[k1]=k+1;
 							break;
 						}
@@ -867,7 +868,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 #endif
 						if(i<k) {
 							for(k3=other_ptr[i];k3<n_all;k3++) {
-								a=1<<k3;
+								a=LK_ONE<<k3;
 								if(id_array[j].temp[X_PAT]&a)	{
 									for(k4=other_ptr[k];k4<n_all;k4++) if(a_set[j][k4]&a)	{
 										gt_store[k1]=k3+1;
@@ -886,7 +887,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 							for(k4=other_ptr[k];k4<n_all;k4++) {
 								a=a_set[j][k4];
 								if(!a) continue;
-								for(k3=other_ptr[i];k3<n_all;k3++) if(a&(1<<k3)) {
+								for(k3=other_ptr[i];k3<n_all;k3++) if(a&(LK_ONE<<k3)) {
 									gt_store[k1]=k3+1;
 									gt_store[k2]=k4+1;
 									break;
@@ -910,7 +911,7 @@ double loki_complex_peelop(const struct Complex_Element *element,const int locus
 					if((other_ptr[i]=k)==n_all) ef=1;
 				} else {
 					a=id_array[j].temp[X_PAT];
-					for(k=other_ptr[i];k<n_all;k++) if(a&(1<<k)) {
+					for(k=other_ptr[i];k<n_all;k++) if(a&(LK_ONE<<k)) {
 						gt_store[k1]=k+1;
 						break;
 					}
@@ -1128,7 +1129,7 @@ double loki_trait_complex_peelop(const struct Complex_Element *element,const int
 			for(k=n_ind-1;k>=0;k--)	{
 				k1=rf[j].id_list[k];
 				if(!gt_store[k1])	{
-					k2|=1<<k1;
+					k2|=LK_ONE<<k1;
 					k3*=n_all;
 					gt_store[k1]=1;
 				}
