@@ -33,16 +33,16 @@
 #include "loki_peel.h"
 #include "loki_output.h"
 #include "sample_rand.h"
+#include "loki_ibd.h"
 #include "loki_dump.h"
 
 #ifdef FUNC_NAME
 #undef FUNC_NAME
 #endif
 #define FUNC_NAME "read_dump"
-int read_dump(int *lp,int *lp1,long *old_pos,int *flag,loki_time *lt)
+int read_dump(int *lp,int *lp1,int *n_ibd,long *old_pos,int *flag,int analysis,loki_time *lt)
 {
 	int i=0,j,k,k1,k2,nl=0,type,nrec,v2[4],pflag=0;
-	unsigned int ui;
 	char *tmp,*tmp1;
 	double v1[4],z;
 	FILE *fdump;
@@ -59,146 +59,152 @@ int read_dump(int *lp,int *lp1,long *old_pos,int *flag,loki_time *lt)
 	} else fdump=fopen(Dumpfile,"r");
 	if(!fdump || i) (void)fprintf(stderr,"[%s:%d] %s(): File Error.  Couldn't open '%s' for input\n",__FILE__,__LINE__,FUNC_NAME,Dumpfile);
 	else {
-		(void)fputs("[Parameters] ",stdout);
-		(void)fflush(stdout);	
 		if(!(tmp=fget_line(fdump))) i=__LINE__;
 		if(!i && strncmp(tmp,"Loki.dump:",10)) i=__LINE__;
-		if(!i) { 
-			ui=strtoul(tmp+10,&tmp1,16);
-			if(ui!=RunID) {
-				i=__LINE__;
-				(void)fputs("!Run ID mismatch! ",stdout);
-			} else for(j=0;!i && j<4;j++) {
-				if(*tmp1++!=',') i=__LINE__;
+		if(!i) {
+			tmp1=tmp+10;
+			for(j=0;!i && j<2;j++) {
+				if(j && (*tmp1++!=',')) i=__LINE__;
 				else v2[j]=strtol(tmp1,&tmp,16);
 				tmp1=tmp;
 			}
 		}
 		if(!i) {
 			*lp=(int)v2[0];
-			nl=(int)v2[1];
-			n_cov_columns=(int)v2[3];
-			k=3;
-			if(polygenic_flag) k++;
-			for(j=0;!i && j<k;j++) {
+			*lp1=(int)v2[1];
+		}
+		if(!i && (analysis&IBD_ANALYSIS)) {
+			if(analysis&ESTIMATE_IBD) i=read_ibd_dump(fdump,n_ibd,tmp);
+			else if(*tmp!='\n') i=__LINE__;
+		}
+		(void)fputs("[Parameters] ",stdout);
+		(void)fflush(stdout);
+		if(!(analysis&IBD_ANALYSIS)) {
+			for(j=0;!i && j<3;j++) {
 				if(*tmp1++!=',') i=__LINE__;
-				if(txt_get_double(tmp1,&tmp,v1+j)) i=__LINE__;
+				else v2[j]=strtol(tmp1,&tmp,16);
 				tmp1=tmp;
 			}
-			for(j=0;!i && j<n_random;j++) {
-				if(*tmp1++!=',') i=__LINE__;
-				if(txt_get_double(tmp1,&tmp,c_var[j])) i=__LINE__;
-				tmp1=tmp;
-			}
-		}
-		if(!i) {
-			residual_var[0]=v1[0];
-			tau[0]=v1[1];
-			grand_mean[0]=v1[2];
-			if(polygenic_flag) additive_var[0]=v1[3];
-			if(*tmp!='\n') i=__LINE__;
-			else if(!(tmp=fget_line(fdump))) i=__LINE__;
-		}
-		for(j=0;!i && j<models[0].n_terms;j++) {
-			type=models[0].term[j].vars[0].type;
-			if(type&(ST_TRAITLOCUS|ST_ID)) continue;
-			for(k=0;!i && k<models[0].term[j].df;k++) {
-				if(txt_get_double(tmp,&tmp1,models[0].term[j].eff+k)) i=__LINE__;
-				if(*tmp1=='\n') {
-					if(!(tmp=fget_line(fdump))) i=__LINE__;
-				} else {
+			if(!i) {
+				nl=(int)v2[0];
+				n_cov_columns=(int)v2[2];
+				k=3;
+				if(polygenic_flag) k++;
+				for(j=0;!i && j<k;j++) {
 					if(*tmp1++!=',') i=__LINE__;
-					tmp=tmp1;
-				}
-			}
-		}
-		*lp1=*lp;
-		*old_pos=-1;
-		*flag=0;
-		if(!i) {
-			if(strncmp(tmp,"LKST:",5)) {
-				*lp1=strtol(tmp,&tmp1,16);
-				if(*tmp1==',') {
-					*old_pos=strtol(++tmp1,&tmp,10);
+					if(txt_get_double(tmp1,&tmp,v1+j)) i=__LINE__;
 					tmp1=tmp;
 				}
-				if(*tmp1==',') {
-					*flag=(int)strtol(++tmp1,&tmp,16);
+				for(j=0;!i && j<n_random;j++) {
+					if(*tmp1++!=',') i=__LINE__;
+					if(txt_get_double(tmp1,&tmp,c_var[j])) i=__LINE__;
 					tmp1=tmp;
 				}
-				if(*tmp1!='\n') i=__LINE__;
-				if(!i && !(tmp=fget_line(fdump))) i=__LINE__;
-			}
-		}
-		if(!i && strncmp(tmp,"LKST:",5)) i=__LINE__;
-		else if(!i) {
-			j=(int)strtol(tmp+5,&tmp1,16);
-			if(j!=N_MOVE_STATS) i=__LINE__;
-		}
-		for(j=0;!i && j<N_MOVE_STATS;j++) {
-			if(*tmp1++!=',') i=__LINE__;
-			else move_stats[j].success=(int)strtol(tmp1,&tmp,16);
-			if(!i && *tmp++!=',') i=__LINE__;
-			else move_stats[j].n=(int)strtol(tmp,&tmp1,16);
-		}
-		if(!i) {
-			if(*tmp1!='\n') i=__LINE__;
-			else if(!(tmp=fget_line(fdump))) i=__LINE__;
-		}
-		if(!i) {
-			if(strncmp(tmp,"LKQT:",5)) i=__LINE__;
-			else {
-				j=(int)strtol(tmp+5,&tmp1,16);
-				if(*tmp1!='\n' || j!=nl) i=__LINE__;
-			}
-		}
-		for(j=0;!i && j<nl;j++) {
-			if(!(tmp=fget_line(fdump))) i=__LINE__;
-			else {
-				k1=(int)strtol(tmp,&tmp1,16);
-				if(k1!=2) ABT_FUNC("Illegal no. alleles for QTL\n");
-				k=get_new_traitlocus(k1);
-				tlocus[k].model_flag=1;
-				if(*tmp1++!=',') i=__LINE__;
-				else {
-					tlocus[k].locus.link_group=(int)strtol(tmp1,&tmp,10);
-					if(*tmp++!=',') i=__LINE__;
-					else tlocus[k].locus.flag=(int)strtol(tmp,&tmp1,16);
-				}
-				if(k<0) ABT_FUNC("Internal error - Couldn't get trait locus\n");
-			}
-			for(k2=0;!i && k2<2;k2++) {
-				if(*tmp1++!=',') i=__LINE__;
-				else {
-					if(txt_get_double(tmp1,&tmp,tlocus[k].locus.pos+k2)) i=__LINE__;
-					else tmp1=tmp;
-				}
 			}
 			if(!i) {
-				k1=k1*(k1+1)/2-1;
-				for(k2=0;!i && k2<k1;k2++) {
+				residual_var[0]=v1[0];
+				tau[0]=v1[1];
+				grand_mean[0]=v1[2];
+				if(polygenic_flag) additive_var[0]=v1[3];
+				if(*tmp!='\n') i=__LINE__;
+				else if(!(tmp=fget_line(fdump))) i=__LINE__;
+			}
+			for(j=0;!i && j<models[0].n_terms;j++) {
+				type=models[0].term[j].vars[0].type;
+				if(type&(ST_TRAITLOCUS|ST_ID)) continue;
+				for(k=0;!i && k<models[0].term[j].df;k++) {
+					if(txt_get_double(tmp,&tmp1,models[0].term[j].eff+k)) i=__LINE__;
+					if(*tmp1=='\n') {
+						if(!(tmp=fget_line(fdump))) i=__LINE__;
+					} else {
+						if(*tmp1++!=',') i=__LINE__;
+						tmp=tmp1;
+					}
+				}
+			}
+			*old_pos=-1;
+			*flag=0;
+			if(!i) {
+				if(strncmp(tmp,"LKST:",5)) {
+					*old_pos=strtol(tmp,&tmp1,10);
+					if(*tmp1==',') {
+						*flag=(int)strtol(++tmp1,&tmp,16);
+						tmp1=tmp;
+					}
+					if(*tmp1!='\n') i=__LINE__;
+					if(!i && !(tmp=fget_line(fdump))) i=__LINE__;
+				}
+			}
+			if(!i && strncmp(tmp,"LKST:",5)) i=__LINE__;
+			else if(!i) {
+				j=(int)strtol(tmp+5,&tmp1,16);
+				if(j!=N_MOVE_STATS) i=__LINE__;
+			}
+			for(j=0;!i && j<N_MOVE_STATS;j++) {
+				if(*tmp1++!=',') i=__LINE__;
+				else move_stats[j].success=(int)strtol(tmp1,&tmp,16);
+				if(!i && *tmp++!=',') i=__LINE__;
+				else move_stats[j].n=(int)strtol(tmp,&tmp1,16);
+			}
+			if(!i) {
+				if(*tmp1!='\n') i=__LINE__;
+				else if(!(tmp=fget_line(fdump))) i=__LINE__;
+			}
+			if(!i) {
+				if(strncmp(tmp,"LKQT:",5)) i=__LINE__;
+				else {
+					j=(int)strtol(tmp+5,&tmp1,16);
+					if(*tmp1!='\n' || j!=nl) i=__LINE__;
+				}
+			}
+			for(j=0;!i && j<nl;j++) {
+				if(!(tmp=fget_line(fdump))) i=__LINE__;
+				else {
+					k1=(int)strtol(tmp,&tmp1,16);
+					if(k1!=2) ABT_FUNC("Illegal no. alleles for QTL\n");
+					k=get_new_traitlocus(k1);
+					tlocus[k].model_flag=1;
 					if(*tmp1++!=',') i=__LINE__;
 					else {
-						if(txt_get_double(tmp1,&tmp,tlocus[k].eff[0]+k2)) i=__LINE__;
+						tlocus[k].locus.link_group=(int)strtol(tmp1,&tmp,10);
+						if(*tmp++!=',') i=__LINE__;
+						else tlocus[k].locus.flag=(int)strtol(tmp,&tmp1,16);
+					}
+					if(k<0) ABT_FUNC("Internal error - Couldn't get trait locus\n");
+				}
+				for(k2=0;!i && k2<2;k2++) {
+					if(*tmp1++!=',') i=__LINE__;
+					else {
+						if(txt_get_double(tmp1,&tmp,tlocus[k].locus.pos+k2)) i=__LINE__;
 						else tmp1=tmp;
 					}
 				}
-			}
-			for(k1=0;!i && k1<n_genetic_groups;k1++) {
-				z=1.0;
-				for(k2=0;!i && k2<tlocus[j].locus.n_alleles-1;k2++) {
-					if(*tmp1++!=',') i=__LINE__;
-					else {
-						if(txt_get_double(tmp1,&tmp,tlocus[k].locus.freq[k1]+k2)) i=__LINE__;
+				if(!i) {
+					k1=k1*(k1+1)/2-1;
+					for(k2=0;!i && k2<k1;k2++) {
+						if(*tmp1++!=',') i=__LINE__;
 						else {
-							z-=tlocus[k].locus.freq[k1][k2];
-							tmp1=tmp;
+							if(txt_get_double(tmp1,&tmp,tlocus[k].eff[0]+k2)) i=__LINE__;
+							else tmp1=tmp;
 						}
 					}
 				}
-				if(!i) tlocus[k].locus.freq[k1][k2]=z;
+				for(k1=0;!i && k1<n_genetic_groups;k1++) {
+					z=1.0;
+					for(k2=0;!i && k2<tlocus[j].locus.n_alleles-1;k2++) {
+						if(*tmp1++!=',') i=__LINE__;
+						else {
+							if(txt_get_double(tmp1,&tmp,tlocus[k].locus.freq[k1]+k2)) i=__LINE__;
+							else {
+								z-=tlocus[k].locus.freq[k1][k2];
+								tmp1=tmp;
+							}
+						}
+					}
+					if(!i) tlocus[k].locus.freq[k1][k2]=z;
+				}
+				if(!i && *tmp1!='\n') i=__LINE__;
 			}
-			if(!i && *tmp1!='\n') i=__LINE__;
 		}
 		if(!i) {
 			if(!(tmp=fget_line(fdump))) i=__LINE__;
@@ -305,7 +311,7 @@ int read_dump(int *lp,int *lp1,long *old_pos,int *flag,loki_time *lt)
 					}
 					if(!i && *tmp!='\n') i=__LINE__;
 				}
-				for(k=j=0;!i && j<n_markers;j++) if(marker[j].mterm[0]) k++;
+				for(k=j=0;!i && j<n_markers;j++) if(marker[j].mterm && marker[j].mterm[0]) k++;
 				if(k) {
 					if(!(tmp=fget_line(fdump))) i=__LINE__;
 					else if(strncmp(tmp,"LKMG:",5)) i=__LINE__;
@@ -336,56 +342,56 @@ int read_dump(int *lp,int *lp1,long *old_pos,int *flag,loki_time *lt)
 				}
 			}
 		}
-		if(!i) {
+		if(!i && !(analysis&IBD_ANALYSIS)) {
 			(void)fputs("[Residuals] ",stdout);
 			(void)fflush(stdout);
 			if(!(tmp=fget_line(fdump))) i=__LINE__;
 			else if(strncmp(tmp,"LKRS:",5)) i=__LINE__;
 			else k2=(int)strtol(tmp+5,&tmp1,16);
 			if(*tmp1++!='\n' || k2!=censored_flag) i=__LINE__;
-		}
-		for(j=0;!i && j<ped_size;j++) {
-			if(!i && polygenic_flag) {
-				if(!(tmp=fget_line(fdump))) i=__LINE__;
-				if(txt_get_double(tmp,&tmp1,&id_array[j].bv[0])) i=__LINE__;
-				if(!i) {
-					if(*tmp1=='\n') {
-						if(pflag==1) i=__LINE__;
-						else pflag=2;
-					} else {
-						if(!i && *tmp1++!=',') i=__LINE__;
-						if(pflag==2) i=__LINE__;
-						pflag=1;
-						if(txt_get_double(tmp1,&tmp,&id_array[j].bvsum[0])) i=__LINE__;
-						if(!i && *tmp++!=',') i=__LINE__;
-						if(txt_get_double(tmp,&tmp1,&id_array[j].bvsum2[0])) i=__LINE__;
-						if(!i && *tmp1!='\n') i=__LINE__;
+			for(j=0;!i && j<ped_size;j++) {
+				if(!i && polygenic_flag) {
+					if(!(tmp=fget_line(fdump))) i=__LINE__;
+					if(txt_get_double(tmp,&tmp1,&id_array[j].bv[0])) i=__LINE__;
+					if(!i) {
+						if(*tmp1=='\n') {
+							if(pflag==1) i=__LINE__;
+							else pflag=2;
+						} else {
+							if(!i && *tmp1++!=',') i=__LINE__;
+							if(pflag==2) i=__LINE__;
+							pflag=1;
+							if(txt_get_double(tmp1,&tmp,&id_array[j].bvsum[0])) i=__LINE__;
+							if(!i && *tmp++!=',') i=__LINE__;
+							if(txt_get_double(tmp,&tmp1,&id_array[j].bvsum2[0])) i=__LINE__;
+							if(!i && *tmp1!='\n') i=__LINE__;
+						}
 					}
 				}
-			}
-			if(!id_array[j].res[0]) continue;
-			if(!(tmp=fget_line(fdump))) i=__LINE__;
-			nrec=id_array[j].n_rec;
-			if(!nrec) nrec=1;
-			for(k=0;!i && k<nrec;k++) {
-				if(k && *tmp++!=',') i=__LINE__;
-				else if(txt_get_double(tmp,&tmp1,id_array[j].res[0]+k)) i=__LINE__;
-				if(!i && use_student_t) {
-					if(*tmp1++!=',') i=__LINE__;
-					else if(txt_get_double(tmp1,&tmp,id_array[j].vv[0]+k)) i=__LINE__;
-					tmp1=tmp;
+				if(!id_array[j].res[0]) continue;
+				if(!(tmp=fget_line(fdump))) i=__LINE__;
+				nrec=id_array[j].n_rec;
+				if(!nrec) nrec=1;
+				for(k=0;!i && k<nrec;k++) {
+					if(k && *tmp++!=',') i=__LINE__;
+					else if(txt_get_double(tmp,&tmp1,id_array[j].res[0]+k)) i=__LINE__;
+					if(!i && use_student_t) {
+						if(*tmp1++!=',') i=__LINE__;
+						else if(txt_get_double(tmp1,&tmp,id_array[j].vv[0]+k)) i=__LINE__;
+						tmp1=tmp;
+					}
+					if(!i && censored_flag && id_array[j].cens[0]) {
+						if(*tmp1++!=',') i=__LINE__;
+						else if(txt_get_double(tmp1,&tmp,id_array[j].cens[0]+k)) i=__LINE__;
+					} else tmp=tmp1;
 				}
-				if(!i && censored_flag && id_array[j].cens[0]) {
-					if(*tmp1++!=',') i=__LINE__;
-					else if(txt_get_double(tmp1,&tmp,id_array[j].cens[0]+k)) i=__LINE__;
-				} else tmp=tmp1;
+				if(!i && *tmp!='\n') i=__LINE__;
+			}
+			if(!i && pflag==1) {
+				if(!(tmp=fget_line(fdump))) i=__LINE__;
+				else bv_iter=(int)strtol(tmp,&tmp1,16);
+				if(!i && *tmp1!='\n') i=__LINE__;
 			}
-			if(!i && *tmp!='\n') i=__LINE__;
-		}
-		if(!i && pflag==1) {
-			if(!(tmp=fget_line(fdump))) i=__LINE__;
-			else bv_iter=(int)strtol(tmp,&tmp1,16);
-			if(!i && *tmp1!='\n') i=__LINE__;
 		}
 		if(!i) {
 			(void)fputs("[Seed] ",stdout);
@@ -431,9 +437,9 @@ int read_dump(int *lp,int *lp1,long *old_pos,int *flag,loki_time *lt)
 #undef FUNC_NAME
 #endif
 #define FUNC_NAME "write_dump"
-void write_dump(int lp,int lp1,long old_pos,int flag,loki_time *lt)
+void write_dump(int lp,int lp1,int n_ibd,long old_pos,int flag,int analysis,loki_time *lt)
 {
-	int i=0,j,k,k1,k2,nx,nl,type,nrec;
+	int i=0,j,k,k1,k2,nx,nl=0,type,nrec;
 	double ex1,ex2;
 	struct tms tbuf;
 	long tps;
@@ -457,65 +463,72 @@ void write_dump(int lp,int lp1,long old_pos,int flag,loki_time *lt)
 			} else fdump=fopen(Dumpfile,"w");
 			if(!fdump || i) (void)fprintf(stderr,"[%s:%d] %s(): File Error.  Couldn't open '%s' for output\n",__FILE__,__LINE__,FUNC_NAME,Dumpfile);
 			else {
+				if(!i && fprintf(fdump,"Loki.dump:%x,%x",lp,lp1)<0) i=1;
+				if(!i && (analysis&IBD_ANALYSIS)) {
+					if(analysis&ESTIMATE_IBD) i=write_ibd_dump(fdump,n_ibd);
+					else if(fputc('\n',fdump)<0) i=1;
+				}
 				(void)fputs("[Parameters] ",stdout);
 				(void)fflush(stdout);
-				nx=0;
-				if(models) {
-					for(j=0;j<models[0].n_terms;j++) {
-						type=models[0].term[j].vars[0].type;
-						if(type&(ST_TRAITLOCUS|ST_ID)) continue;
-						nx+=models[0].term[j].df;
-					}
-				}
-				for(nl=j=0;j<n_tloci;j++) if(tlocus[j].locus.flag) nl++;
-				if(!i && fprintf(fdump,"Loki.dump:%x,%x,%x,%x,%x,",RunID,lp,nl,nx,n_cov_columns)<0) i=1;
-				if(models) {
-					if(!i) i=txt_print_double(residual_var[0],fdump);
-					if(!i && fputc(',',fdump)<0) i=1;
-					if(!i) i=txt_print_double(tau[0],fdump);
-					if(!i && fputc(',',fdump)<0) i=1;
-					if(!i) i=txt_print_double(grand_mean[0],fdump);
-					if(!i && polygenic_flag) {
-						if(!i && fputc(',',fdump)<0) i=1;
-						i=txt_print_double(additive_var[0],fdump);
-					}
-					for(j=0;!i && j<n_random;j++) {
-						if(!i && fputc(',',fdump)<0) i=1;
-						i=txt_print_double(c_var[j][0],fdump);
-					}
-					if(!i && fputc('\n',fdump)<0) i=1;
-					for(j=0;!i && j<models[0].n_terms;j++) {
-						type=models[0].term[j].vars[0].type;
-						if(type&(ST_TRAITLOCUS|ST_ID)) continue;
-						for(k=0;!i && k<models[0].term[j].df;k++) {
-							i=txt_print_double(models[0].term[j].eff[k],fdump);	
-							if(!i && fputc('\n',fdump)<0) i=1;
+				if(!(analysis&IBD_ANALYSIS)) {
+					nx=0;
+					if(models) {
+						for(j=0;j<models[0].n_terms;j++) {
+							type=models[0].term[j].vars[0].type;
+							if(type&(ST_TRAITLOCUS|ST_ID)) continue;
+							nx+=models[0].term[j].df;
 						}
 					}
-				}
-				if(!i && fprintf(fdump,"%x,%ld,%x",lp1,old_pos,flag)<0) i=1;
-				if(!i && fprintf(fdump,"\nLKST:%x",N_MOVE_STATS)<0) i=1;
-				for(j=0;!i && j<N_MOVE_STATS;j++) if(fprintf(fdump,",%x,%x",move_stats[j].success,move_stats[j].n)<0) i=1;
-				if(!i && fprintf(fdump,"\nLKQT:%x\n",nl)<0) i=1;
-				for(j=0;!i && j<n_tloci;j++) if(tlocus[j].locus.flag) {
-					if(fprintf(fdump,"%x,%d,%x",tlocus[j].locus.n_alleles,tlocus[j].locus.link_group,tlocus[j].locus.flag)<0) i=1;
-					for(k=0;!i && k<2;k++) {
-						if(fputc(',',fdump)<0) i=1;
-						else i=txt_print_double(tlocus[j].locus.pos[k],fdump);	
+					for(nl=j=0;j<n_tloci;j++) if(tlocus[j].locus.flag) nl++;
+					if(!i && fprintf(fdump,",%x,%x,%x,",nl,nx,n_cov_columns)<0) i=1;
+					if(models) {
+						if(!i) i=txt_print_double(residual_var[0],fdump);
+						if(!i && fputc(',',fdump)<0) i=1;
+						if(!i) i=txt_print_double(tau[0],fdump);
+						if(!i && fputc(',',fdump)<0) i=1;
+						if(!i) i=txt_print_double(grand_mean[0],fdump);
+						if(!i && polygenic_flag) {
+							if(!i && fputc(',',fdump)<0) i=1;
+							i=txt_print_double(additive_var[0],fdump);
+						}
+						for(j=0;!i && j<n_random;j++) {
+							if(!i && fputc(',',fdump)<0) i=1;
+							i=txt_print_double(c_var[j][0],fdump);
+						}
+						if(!i && fputc('\n',fdump)<0) i=1;
+						for(j=0;!i && j<models[0].n_terms;j++) {
+							type=models[0].term[j].vars[0].type;
+							if(type&(ST_TRAITLOCUS|ST_ID)) continue;
+							for(k=0;!i && k<models[0].term[j].df;k++) {
+								i=txt_print_double(models[0].term[j].eff[k],fdump);	
+								if(!i && fputc('\n',fdump)<0) i=1;
+							}
+						}
 					}
-					k=tlocus[j].locus.n_alleles;
-					k1=k*(k+1)/2-1;
-					for(k=0;!i && k<k1;k++) {
-						if(fputc(',',fdump)<0) i=1;
-						else i=txt_print_double(tlocus[j].eff[0][k],fdump);
-					}
-					for(k=0;!i && k<n_genetic_groups;k++) {
-						for(k1=0;!i && k1<tlocus[j].locus.n_alleles-1;k1++) {
+					if(!i && fprintf(fdump,"%ld,%x",old_pos,flag)<0) i=1;
+					if(!i && fprintf(fdump,"\nLKST:%x",N_MOVE_STATS)<0) i=1;
+					for(j=0;!i && j<N_MOVE_STATS;j++) if(fprintf(fdump,",%x,%x",move_stats[j].success,move_stats[j].n)<0) i=1;
+					if(!i && fprintf(fdump,"\nLKQT:%x\n",nl)<0) i=1;
+					for(j=0;!i && j<n_tloci;j++) if(tlocus[j].locus.flag) {
+						if(fprintf(fdump,"%x,%d,%x",tlocus[j].locus.n_alleles,tlocus[j].locus.link_group,tlocus[j].locus.flag)<0) i=1;
+						for(k=0;!i && k<2;k++) {
 							if(fputc(',',fdump)<0) i=1;
-							else i=txt_print_double(tlocus[j].locus.freq[k][k1],fdump);
+							else i=txt_print_double(tlocus[j].locus.pos[k],fdump);	
 						}
+						k=tlocus[j].locus.n_alleles;
+						k1=k*(k+1)/2-1;
+						for(k=0;!i && k<k1;k++) {
+							if(fputc(',',fdump)<0) i=1;
+							else i=txt_print_double(tlocus[j].eff[0][k],fdump);
+						}
+						for(k=0;!i && k<n_genetic_groups;k++) {
+							for(k1=0;!i && k1<tlocus[j].locus.n_alleles-1;k1++) {
+								if(fputc(',',fdump)<0) i=1;
+								else i=txt_print_double(tlocus[j].locus.freq[k][k1],fdump);
+							}
+						}
+						if(!i && fputc('\n',fdump)<0) i=1;
 					}
-					if(!i && fputc('\n',fdump)<0) i=1;
 				}
 				if(!i && fprintf(fdump,"LKMK:%x\n",n_markers)<0) i=1;
 				for(j=0;!i && j<n_markers;j++) {
@@ -582,38 +595,38 @@ void write_dump(int lp,int lp1,long old_pos,int flag,loki_time *lt)
 						}
 					}
 				}
-				if(!i) {
+				if(!i && !(analysis&IBD_ANALYSIS)) {
 					(void)fputs("[Residuals] ",stdout);
 					(void)fflush(stdout);
 					if(fprintf(fdump,"LKRS:%x\n",censored_flag)<0) i=1;
-				}
-				if(models) for(j=0;!i && j<ped_size;j++) {
-					if(polygenic_flag) {
-						i=txt_print_double(id_array[j].bv[0],fdump);
-						if(!i && fputc(',',fdump)<0) i=1;
-						if(!i) i=txt_print_double(id_array[j].bvsum[0],fdump);
-						if(!i && fputc(',',fdump)<0) i=1;
-						if(!i) i=txt_print_double(id_array[j].bvsum2[0],fdump);
+					if(models) for(j=0;!i && j<ped_size;j++) {
+						if(polygenic_flag) {
+							i=txt_print_double(id_array[j].bv[0],fdump);
+							if(!i && fputc(',',fdump)<0) i=1;
+							if(!i) i=txt_print_double(id_array[j].bvsum[0],fdump);
+							if(!i && fputc(',',fdump)<0) i=1;
+							if(!i) i=txt_print_double(id_array[j].bvsum2[0],fdump);
+							if(!i && fputc('\n',fdump)<0) i=1;
+						}
+						if(!id_array[j].res[0]) continue;
+						nrec=id_array[j].n_rec;
+						if(!nrec) nrec=1;
+						for(k=0;!i && k<nrec;k++) {
+							if(k && fputc(',',fdump)<0) i=1;
+							else i=txt_print_double(id_array[j].res[0][k],fdump);
+							if(!i && use_student_t) {
+								if(fputc(',',fdump)<0) i=1;
+								else i=txt_print_double(id_array[j].vv[0][k],fdump);
+							}
+							if(!i && censored_flag && id_array[j].cens[0]) {
+								if(fputc(',',fdump)<0) i=1;
+								else i=txt_print_double(id_array[j].cens[0][k],fdump);
+							}
+						}
 						if(!i && fputc('\n',fdump)<0) i=1;
 					}
-					if(!id_array[j].res[0]) continue;
-					nrec=id_array[j].n_rec;
-					if(!nrec) nrec=1;
-					for(k=0;!i && k<nrec;k++) {
-						if(k && fputc(',',fdump)<0) i=1;
-						else i=txt_print_double(id_array[j].res[0][k],fdump);
-						if(!i && use_student_t) {
-							if(fputc(',',fdump)<0) i=1;
-							else i=txt_print_double(id_array[j].vv[0][k],fdump);
-						}
-						if(!i && censored_flag && id_array[j].cens[0]) {
-							if(fputc(',',fdump)<0) i=1;
-							else i=txt_print_double(id_array[j].cens[0][k],fdump);
-						}
-					}
-					if(!i && fputc('\n',fdump)<0) i=1;
+					if(!i && polygenic_flag) if(fprintf(fdump,"%x\n",bv_iter)<0) i=1;
 				}
-				if(!i && polygenic_flag) if(fprintf(fdump,"%x\n",bv_iter)<0) i=1;
 				if(!i) {
 					(void)fputs("[Seed] ",stdout);
 					(void)fflush(stdout);
