@@ -37,7 +37,8 @@
 #include "loki_ibd.h"
 #include "seg_pen.h"
 
-static double *pos_list,***ibd[2];
+static double *pos_list;
+static unsigned int ***ibd[2];
 static int *loci,*seg[2],*n_longs,*n_pairs,*inbr,**inb_sparse,**inb_sparse1;
 static unsigned long *founders;
 
@@ -183,7 +184,6 @@ int SetupIBD(void)
 {
 	int i,i1,i2,i3,j,k,k1,k2,np,cs,n_long,*sz,*sz1;
 	int ids,idd,ct,ct1,*pp,*pp1;
-	double *tp;
 	unsigned long *tpl,*tpl1,*tpl2;
 	
 	if(!founders) get_founders(&founders,&inbr,&n_longs,&n_pairs);
@@ -211,12 +211,11 @@ int SetupIBD(void)
 			ibd[1]=ibd[0]+j;
 			if(!(ibd[0][0]=malloc(sizeof(void *)*2*j*n_comp))) ABT_FUNC(MMsg);
 			ibd[1][0]=ibd[0][0]+j*n_comp;
-			if(!(ibd[0][0][0]=tp=malloc(sizeof(double)*2*j*np))) {
-				fprintf(stderr,"Asked for 2*%d*%d*sizeof(double)=%d\n",j,np,sizeof(double)*2*j*np);
+			if(!(ibd[0][0][0]=calloc((size_t)2*j*np,sizeof(int)))) {
+				fprintf(stderr,"Asked for 2*n_locations*n_pairs*sizeof(int)=2*%d*%d*%d=%d bytes\n",j,np,sizeof(int),2*j*np*sizeof(int));
 				ABT_FUNC(MMsg);
 			}
 			ibd[1][0][0]=ibd[0][0][0]+j*np;
-			for(i=0;i<2*j*np;i++) tp[i]=0.0;
 			for(i=1;i<j;i++) {
 				for(k1=0;k1<2;k1++) {
 					ibd[k1][i]=ibd[k1][i-1]+n_comp;
@@ -424,8 +423,9 @@ static int get_pos_list(int i,int nl)
 void Handle_IBD(void)
 {
 	int i,i1,i2,i3,i4,j,k,k1,kk,ix=0,*genes[2],nl,s,comp,g0,g1,g2,g3,cs,ids,idd,np,kk1;
-	int *pp;
-	double x,z,ps[2],tp[2],**ibd1[2];
+	int *pp,ct;
+	unsigned int **ibd1[2];
+	double x,z,ps[2],tp[2];
 	
 	genes[0]=loci+n_markers;
 	genes[1]=genes[0]+ped_size;
@@ -478,26 +478,26 @@ void Handle_IBD(void)
 							g3=genes[1][i4];
 							if(g0==g2) {
 								if(g1==g3) {
-									ibd1[1][comp][i3]++;
-									z=(g0==g1)?2.0:1.0;
+									ibd1[1][comp][i3]+=2.0;
+									ct=(g0==g1)?4:2;
 								} else {
-									z=.5;
-									if(g0==g3) z+=.5;
-									if(g1==g2) z+=.5;
+									ct=1;
+									if(g0==g3) ct++;
+									if(g1==g2) ct++;
 								}
 							} else if(g0==g3) {
 								if(g1==g2) {
-									ibd1[1][comp][i3]++;
-									z=1.0;
-								} else z=(g1==g3)?1.0:0.5;
+									ibd1[1][comp][i3]+=2.0;
+									ct=2;
+								} else ct=(g1==g3)?2:1;
 							} else {
-								z=0.0;
-								if(g1==g2) z+=.5;
-								if(g1==g3) z+=.5;
+								ct=0;
+								if(g1==g2) ct++;
+								if(g1==g3) ct++;
 							}
-							ibd1[0][comp][i3++]+=z;
+							ibd1[0][comp][i3++]+=ct;
 						}
-						if(inbr[i1]) ibd1[0][comp][i3++]+=(g0==g1)?2.0:1.0;
+						if(inbr[i1]) ibd1[0][comp][i3++]+=(g0==g1)?4:2;
 					}
 				}
 			}
@@ -562,15 +562,21 @@ double score_ibd(int locus,int *k5,int comp,int affn,int *aff,double *pairs,int
  	return l; 
 }
 
-static void print_pair(FILE *fptr,int i,int j)
+static int print_pair(FILE *fptr,int i,int j)
 {
-	if(family_id) {
-		print_orig_family(fptr,i,0);
+	int k=0;
+	
+	if(has_orig_id(i) && has_orig_id(j)) {
+		if(family_id) {
+			print_orig_family(fptr,i,0);
+			(void)fputc(' ',fptr);
+		}
+		print_orig_id1(fptr,i);
 		(void)fputc(' ',fptr);
+		print_orig_id1(fptr,j);
+		k=1;
 	}
-	print_orig_id1(fptr,i);
-	(void)fputc(' ',fptr);
-	print_orig_id1(fptr,j);
+	return k;
 }
 
 #ifdef FUNC_NAME
@@ -610,17 +616,137 @@ static FILE *open_ibd_file(char *name,int md)
 	return fptr;
 }
   
+int read_ibd_dump(FILE *fdump,int *n_ibd,char *tmp)
+{
+	int link,k,j,k1,k2=0,comp,i=0;
+	char *tmp1;
+	
+	for(k=link=0;link<n_links;link++) if(linkage[link].ibd_est_type) k++;
+	if(k) {
+		(void)fputs("[IBD matrices] ",stdout);
+		(void)fflush(stdout);
+		/* Count no. locations */
+		for(link=j=k1=0;link<n_links;link++) if(linkage[link].ibd_est_type) {
+			k=0;
+			switch(linkage[link].ibd_est_type) {
+			 case IBD_EST_DISCRETE:
+				k=linkage[link].ibd_list->idx;
+				break;
+			 case IBD_EST_MARKERS:
+				k=linkage[link].n_markers;
+				break;
+			 case IBD_EST_GRID:
+				k=1+(int)(.5+(linkage[link].ibd_list->pos[1]-linkage[link].ibd_list->pos[0])/linkage[link].ibd_list->pos[2]);
+				break;
+			}
+			if(k>k1) k1=k;
+			j+=k;
+		}
+		if(*tmp++!=',') i=__LINE__;
+		if(!i) {
+			k1=(int)strtol(tmp,&tmp1,16);
+			if(k1!=j) i=__LINE__;
+		}
+		if(!i && *tmp1++!=',') i=__LINE__;
+		if(!i) {
+			*n_ibd=(int)strtol(tmp1,&tmp,16);
+			if(*tmp!='\n') i=__LINE__;
+		}
+		for(comp=0;!i && comp<n_comp;comp++) if(n_pairs[comp]) {
+			if(!(tmp=fget_line(fdump))) i=__LINE__;
+			if(!i && strncmp(tmp,"LKCM:",5)) i=__LINE__;
+			tmp+=5;
+			if(!i) {
+				k=(int)strtol(tmp,&tmp1,16);
+				if(k!=comp) i=__LINE__;
+			}
+			if(!i && *tmp1++!=',') i=__LINE__;
+			if(!i) {
+				k=(int)strtol(tmp1,&tmp,16);
+				if(k!=n_pairs[comp]) i=__LINE__;
+			}
+			if(!i && *tmp!='\n') i=__LINE__;
+			for(k=0;!i && k<n_pairs[comp];k++) {
+				for(k1=0;!i && k1<j;k1++) {
+					if(!(tmp=fget_line(fdump))) i=__LINE__;
+					else k2=(int)strtol(tmp,&tmp1,16);
+					if(!i) {
+						ibd[0][k1][comp][k]=k2;
+						if(*tmp1++!=',') i=__LINE__;
+					}
+					if(!i) {
+						k2=(int)strtol(tmp1,&tmp,16);
+						ibd[1][k1][comp][k]=k2;
+						if(*tmp!='\n') i=__LINE__;
+					}
+				}
+			}
+		}
+	}
+	return -i;
+}
+
+int write_ibd_dump(FILE *fdump,int n_ibd)
+{
+	int link,k,j,k1,k2,comp,i=0;
+	
+	for(k=link=0;link<n_links;link++) if(linkage[link].ibd_est_type) k++;
+	if(k) {
+		(void)fputs("[IBD matrices] ",stdout);
+		(void)fflush(stdout);
+		/* Count no. locations */
+		for(link=j=k1=0;link<n_links;link++) if(linkage[link].ibd_est_type) {
+			k=0;
+			switch(linkage[link].ibd_est_type) {
+			 case IBD_EST_DISCRETE:
+				k=linkage[link].ibd_list->idx;
+				break;
+			 case IBD_EST_MARKERS:
+				k=linkage[link].n_markers;
+				break;
+			 case IBD_EST_GRID:
+				k=1+(int)(.5+(linkage[link].ibd_list->pos[1]-linkage[link].ibd_list->pos[0])/linkage[link].ibd_list->pos[2]);
+				break;
+			}
+			if(k>k1) k1=k;
+			j+=k;
+		}
+		if(fprintf(fdump,",%x,%x\n",j,n_ibd)<0) i=1;
+		for(comp=0;!i && comp<n_comp;comp++) if(n_pairs[comp]) {
+			if(fprintf(fdump,"LKCM:%x,%x\n",comp,n_pairs[comp])<0) i=1;
+			for(k=0;!i && k<n_pairs[comp];k++) {
+				for(k1=0;!i && k1<j;k1++) {
+					k2=ibd[0][k1][comp][k];
+					if(k2) {
+						if(fprintf(fdump,"%x,",k2)<0) i=1;
+					} else {
+						if(fputc(',',fdump)<0) i=1;
+					}
+					k2=ibd[1][k1][comp][k];
+					if(k2) {
+						if(fprintf(fdump,"%x\n",k2)<0) i=1;
+					} else {
+						if(fputc('\n',fdump)<0) i=1;
+					}
+				}
+			}
+		}
+	}
+	return i;
+}
+
 void Output_Sample_IBD(int lp,int ibd_md)
 {
 	int i,i1,i2,j,k,k1,l,link,comp,n_long,ids,idd,np,nl=0;
-	double *ibd1[2],z;
+	double z;
+	unsigned int *ibd1[2];
 	unsigned long *tp,*tp1,*tp2;
 	FILE *fptr;
 	
 	if(lp<1) return;
 	fptr=open_ibd_file(OutputIBDfile,ibd_md);
 	(void)fprintf(fptr,"Iteration %d\n",lp);
-	z=1.0/(double)lp;
+	z=0.5/(double)lp;
 	for(k=link=0;link<n_links;link++) {
 		if(linkage[link].ibd_est_type) {
 			(void)fprintf(fptr,"\n**Linkage group %s:\n",linkage[link].name);
@@ -645,19 +771,19 @@ void Output_Sample_IBD(int lp,int ibd_md)
 							for(k1=0;k1<n_long;k1++) if(tp[k1]&tp1[k1]) break;
 							if(k1==n_long) continue;
 							if(!(inbr[i1+i]||inbr[i1+j])&&(ids==j||idd==j)) {
-								print_pair(fptr,i1+i+1,i1+j+1);
-								(void)fputs(" 0.5 0\n",fptr);
+								if(print_pair(fptr,i1+i+1,i1+j+1)) (void)fputs(" 0.5 0\n",fptr);
 							} else {
 								if(ibd1[0][i2]) {
-									print_pair(fptr,i1+i+1,i1+j+1);
-									(void)fprintf(fptr," %g %g\n",ibd1[0][i2]*z,ibd1[1][i2]*z);
+									if(print_pair(fptr,i1+i+1,i1+j+1))
+									  (void)fprintf(fptr," %g %g\n",(double)ibd1[0][i2]*z,(double)ibd1[1][i2]*z);
 								}
 								i2++;
 							}
 						}
-						print_pair(fptr,i1+i+1,i1+i+1);
-						if(inbr[i1+i]) (void)fprintf(fptr," %g 1\n",ibd1[0][i2++]*z);
-						else (void)fputs(" 1 1\n",fptr);
+						if(print_pair(fptr,i1+i+1,i1+i+1)) {
+							if(inbr[i1+i]) (void)fprintf(fptr," %g 1\n",(double)ibd1[0][i2++]*z);
+							else (void)fputs(" 1 1\n",fptr);
+						}
 					}
 					i1+=i;
 				}
@@ -675,13 +801,14 @@ void Output_Sample_IBD(int lp,int ibd_md)
 void Output_Merlin_IBD(int lp,int ibd_md)
 {
 	int i,i1,i2,j,k,k1,l,link,comp,n_long,ids,idd,np=0,nl;
-	double x,*ibd1[2],z,s0,s1,s2;
+	unsigned int *ibd1[2];
+	double x,z,s0,s1,s2;
 	unsigned long *tp,*tp1,*tp2;
 	FILE *fptr;
 	
 	if(lp<1) return;
 	fptr=open_ibd_file(OutputIBDfile,ibd_md);
-	z=1.0/(double)lp;
+	z=0.5/(double)lp;
 	tp=founders;
 	for(i1=comp=0;comp<n_comp-singleton_flag;comp++) {
 		n_long=n_longs[comp];
@@ -706,25 +833,26 @@ void Output_Merlin_IBD(int lp,int ibd_md)
 							for(k1=0;k1<n_long;k1++) if(tp[k1]&tp1[k1]) break;
 							if(k1<n_long) {
 								if(!(inbr[i1+i]||inbr[i1+j])&&(ids==j||idd==j)) {
-									print_pair(fptr,i1+i+1,i1+j+1);
-									(void)fprintf(fptr," %.3f 0.0 1.0 0.0\n",x);
+									if(print_pair(fptr,i1+i+1,i1+j+1))
+									  (void)fprintf(fptr," %.3f 0.0 1.0 0.0\n",x);
 								} else {
-									print_pair(fptr,i1+i+1,i1+j+1);
-									s2=ibd1[1][i2]*z;
-									s1=2.0*z*(ibd1[0][i2]-ibd1[1][i2]);
-									s0=1.0-s1-s2;
-									if(s0<0) s0=0.0;
-									(void)fprintf(fptr," %.3f %.5f %.5f %.5f\n",x,s0,s1,s2);
+									if(print_pair(fptr,i1+i+1,i1+j+1)) {
+										s2=(double)ibd1[1][i2]*z;
+										s1=2.0*z*(ibd1[0][i2]-ibd1[1][i2]);
+										s0=1.0-s1-s2;
+										if(s0<0) s0=0.0;
+										(void)fprintf(fptr," %.3f %.5f %.5f %.5f\n",x,s0,s1,s2);
+									}
 									i2++;
 								}
 							} else {
-								print_pair(fptr,i1+i+1,i1+j+1);
-								(void)fprintf(fptr," %.3f 1.0 0.0 0.0\n",x);
+								if(print_pair(fptr,i1+i+1,i1+j+1))
+								  (void)fprintf(fptr," %.3f 1.0 0.0 0.0\n",x);
 							}
 						}
-						print_pair(fptr,i1+i+1,i1+i+1);
+						if(print_pair(fptr,i1+i+1,i1+i+1))
+						  (void)fprintf(fptr," %.3f 0.0 0.0 1.0\n",x);
 						if(inbr[i1+i]) i2++;
-						(void)fprintf(fptr," %.3f 0.0 0.0 1.0\n",x);
 					}
 					k++;
 				}
@@ -746,8 +874,8 @@ void Output_Merlin_IBD(int lp,int ibd_md)
 					for(i=0;i<comp_size[comp];i++) {
 						ids=id_array[i1+i].sire-1-i1;
 						idd=id_array[i1+i].dam-1-i1;
-						print_pair(fptr,i1+i+1,i1+i+1);
-						(void)fprintf(fptr," %.3f 0.0 0.0 1.0\n",x);
+						if(print_pair(fptr,i1+i+1,i1+i+1))
+						  (void)fprintf(fptr," %.3f 0.0 0.0 1.0\n",x);
 					}
 				}
 			}
@@ -758,14 +886,15 @@ void Output_Merlin_IBD(int lp,int ibd_md)
 
 void Output_Solar_IBD(int lp,int ibd_md,int *trans)
 {
-	int i,i1,i2,j,k,kk,l,link,comp,ids,idd,np=0,nl,dlen,clen,*pp,s1,s2;
-	double x,*ibd1[2],z,z1;
+	int i,i1,i2,j,k,kk,l,link,comp,ids,idd,np=0,nl,dlen,clen,*pp,s1,s2,ct;
+	unsigned int *ibd1[2];
+	double x,z;
 	char *dname,*fname;
 	struct stat sb;
 	FILE *fptr;
 	
 	if(lp<1) return;
-	z=1.0/(double)lp;
+	z=0.5/(double)lp;
 	if(OutputIBDdir) dname=OutputIBDdir;
 	else dname=make_file_name("_ibd");
 	/* Check if output dir exists */
@@ -843,19 +972,19 @@ void Output_Solar_IBD(int lp,int ibd_md,int *trans)
 								else (void)fprintf(fptr,"%5d %5d",s2,s1);
 								(void)fprintf(fptr," %10.7f %10.7f\n",0.5,0.0);
 							} else {
-								z1=ibd1[0][i2];
-								if(z1>0.0) {
+								ct=ibd1[0][i2];
+								if(ct) {
 									if(s1>s2) (void)fprintf(fptr,"%5d %5d",s1,s2);
 									else (void)fprintf(fptr,"%5d %5d",s2,s1);
-									(void)fprintf(fptr," %10.7f %10.7f\n",z1*z,ibd1[1][i2]*z);
+									(void)fprintf(fptr," %10.7f %10.7f\n",(double)ct*z,(double)ibd1[1][i2]*z);
 								}
 								i2++;
 							}
 						}
 						(void)fprintf(fptr,"%5d %5d",trans[i1+i],trans[i1+i]);
 						if(inbr[i1+i]) {
-							z1=ibd1[0][i2++];
-							if(z1>0.0) (void)fprintf(fptr," %10.7f %10.7f\n",z1*z,1.0);
+							ct=ibd1[0][i2++];
+							if(ct) (void)fprintf(fptr," %10.7f %10.7f\n",(double)ct*z,1.0);
 						} else (void)fprintf(fptr," %10.7f %10.7f\n",1.0,1.0);
 					}
 					i1+=comp_size[comp];
