@@ -9,6 +9,8 @@ void Output_Sample_IBD(int,int);
 void Output_Merlin_IBD(int,int);
 void Output_Solar_IBD(int,int,int *);
 void get_founder_params(unsigned long **,int **,int **,int **);
+int write_ibd_dump(FILE *,int);
+int read_ibd_dump(FILE *,int *,char *);
 
 #define IBD_MIN_GRID_STEP .0001
 #define IBD_MAX_GRID 10000
