@@ -15,6 +15,7 @@
  * under the terms of the Modified BSD license, see the file COPYING        *
  *                                                                          *
  ****************************************************************************/
+/* add routines to output QTL vectors for stat5 EWD */
 
 #include <stdlib.h>
 #ifdef USE_DMALLOC
@@ -669,3 +670,44 @@ void OutputHeader(FILE *fptr,int si,loki_time *lt)
 	(void)fflush(fptr);
 }
 
+/* the following routines are for stat 5 EWD */
+#ifdef FUNC_NAME
+#undef FUNC_NAME
+#endif
+#define FUNC_NAME "OutputQTLvect"
+void OutputQTLvect(FILE *qptr,int lp)
+/* June 3, 1998 EWD - output a QTL vector to file */
+/* Jan 12, 2004 EWD - update? for new structure */
+{
+   int l,n;
+   for(l=0;l<n_tloci;l++) if(tlocus[l].locus.flag) {
+      fprintf(qptr,"%d ",lp);
+      if(tlocus[l].locus.flag&TL_LINKED) {
+         fprintf(qptr,"%d ",tlocus[l].locus.link_group+1);
+      }
+      else fputs("0 ",qptr);
+      for(n=0;n<ped_size;n++) fprintf(qptr,"%d ",tlocus[l].locus.gt[n]);
+      fputc('\n',qptr);
+   }
+}
+
+#ifdef FUNC_NAME
+#undef FUNC_NAME
+#endif
+#define FUNC_NAME "OutputQTLHead"
+void OutputQTLHead(FILE *qptr)
+/* June 3, 1998 EWD - print header information for QTL genotype vector file */
+{
+   int i;
+   fprintf(qptr," %d %d\n",ped_size,n_comp);
+   for(i=0;i<ped_size;i++) {
+      print_orig_id(qptr,i+1);
+      fputc(' ',qptr);
+   }
+   fputc('\n',qptr);
+   for(i=0;i<ped_size;i++) {
+      fprintf(qptr,"%d ",id_array[i].comp);
+   }
+   fputc('\n',qptr);
+}
+
