@@ -277,7 +277,7 @@ double peel_locus(const int *perm,int idx,int n_loci,int sample_flag,struct peel
 			for(grp=0;grp<n_genetic_groups;grp++)
 			  for(i=0;i<nn_all;i++) freq[grp][peel_alls[i]]+=loc->freq[grp][i];
 			lump=0;
-			for(i=0;i<nn_all;i++) if(peel_alls[i]==n_all-1) lump|=(1<<i);
+			for(i=0;i<nn_all;i++) if(peel_alls[i]==n_all-1) lump|=(LK_ONE<<i);
 		}
 		n_peel_ops=0;
 		cs=comp_size[comp];
@@ -522,7 +522,7 @@ double peel_locus(const int *perm,int idx,int n_loci,int sample_flag,struct peel
 							}
 #endif
 							a=req_set[k1][locus][i1];
-							if(a&(1<<allele)) {
+							if(a&(LK_ONE<<allele)) {
 								z=0.0;
 								k=0;
 								while(a) {
@@ -682,11 +682,11 @@ double peel_locus(const int *perm,int idx,int n_loci,int sample_flag,struct peel
 #ifdef DEBUG
 						if(k<0 || k>=n_all) ABT_FUNC("Internal error - bad sampled genotype\n");
 #endif
-						a=1<<k;
+						a=LK_ONE<<k;
 						if(a&c) a=c;
 						for(k1=0;k1<2;k1++) {
 							k=id_array[idd-1].allele[k1]-1;
-							b=1<<k;
+							b=LK_ONE<<k;
 							trn[k1]=(a&b)?1:0;
 						}
 #ifdef DEBUG
@@ -709,11 +709,11 @@ double peel_locus(const int *perm,int idx,int n_loci,int sample_flag,struct peel
 #ifdef DEBUG
 						if(k<0 || k>=n_all) ABT_FUNC("Internal error - bad sampled genotype\n");
 #endif
-						a=1<<k;
+						a=LK_ONE<<k;
 						if(a&c) a=c;
 						for(k1=0;k1<2;k1++) {
 							k=id_array[ids-1].allele[k1]-1;
-							b=1<<k;
+							b=LK_ONE<<k;
 							trn[k1]=(a&b)?1:0;
 						}
 #ifdef DEBUG
