@@ -15,6 +15,7 @@
  * under the terms of the Modified BSD license, see the file COPYING        *
  *                                                                          *
  ****************************************************************************/
+/* Jan 9 2004 attempted to add stat 5 code EWD */
 
 #include <config.h>
 #include <stdlib.h>
@@ -65,7 +66,7 @@
 #include "read_solar_idfile.h"
 
 double *tau_beta,*tau,lm_ratio;
-int no_overdominant,tau_mode,analysis=DEFAULT_ANALYSIS;
+int no_overdominant,tau_mode,analysis=DEFAULT_ANALYSIS,genv_out; /* stat5 EWD */
 struct lmon_param *lpar;
 
 #ifdef FUNC_NAME
@@ -75,11 +76,11 @@ struct lmon_param *lpar;
 void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int append_output_flag,loki_time *lt)
 {
 	int lp,i,j,j1,j2,k,k1,k2,flag=0,comp,*perm=0,*naffect=0,**affs=0,dumped=0,ibdflag,*ntl_linked=0,*ntl_linked1=0,lp1;
-	FILE *fptr=0,*ffreq=0,*fpos=0,*fmpos=0;
+	FILE *fptr=0,*ffreq=0,*fpos=0,*fmpos=0,*qptr=0; /* EWD */
  	double z,*ss=0,*ss2=0,**pairs=0,*trpos=0,*trpos1=0;
 	struct output_gen *og;
 	double addlog(double x1,double x2);
-	char *mposname=0;
+	char *mposname=0,*qname=0; /* EWD */
 	int *mpos_perm=0,*solar_trans,n_ibd=0;
 	long old_pos=-1;
 #ifdef USE_MONITOR
@@ -129,21 +130,15 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 		/* Yes - read in dump file */
 		(void)fputs("Retrieving program state: ",stdout);
 		(void)fflush(stdout);
-		j=read_dump(&lp,&lp1,&old_pos,&flag,lt);
+		j=read_dump(&lp,&lp1,&n_ibd,&old_pos,&flag,analysis,lt);
 		if(j<0) {
 			(void)fputs("FAILED\n",stdout);
 			ABT_FUNC(AbMsg);
 		} else (void)fputs("OK\n",stdout);
+		flag&=~6; /* Need to re-do founder genes and calculate seg-probs */
 	} else {
 		/* No - Get initial genotype samples for all markers */
 		if(!(analysis&NULL_ANALYSIS)) for(k=0;k<n_links;k++) Sample_LinkageGroup(k,peel_work,*si_mode);
-/*		k1=0;
-		z=peel_locus(&k1,0,1,0,peel_work,*si_mode);
-		printf("%s %g\n",marker[k1].name,z);
-		k1++;
-		z=peel_locus(&k1,0,1,0,peel_work,*si_mode);
-		printf("%s %g\n",marker[k1].name,z);
-		exit(0); */
 		if(!analysis) {
 			/* Quantitative analysis */
 			if(!tlocus) start_tloci=0;
@@ -210,6 +205,10 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 			if(!OutputPosfile) OutputPosfile=make_file_name(".pos");
 		}
 	}
+	if(genv_out) { /* June 2, 1998 EWD - open file to put genotype vectors into */
+		qname=make_file_name(".qtl");
+		if(!(qptr=fopen(qname,"w"))) abt(__FILE__,__LINE__,"%s(): File Error.  Couldn't open '%s' for output\n",FUNC_NAME,qname);
+	}
 	for(i=0;i<n_markers;i++) if(marker[i].pos_set==2) break;
 	if(i<n_markers) {
 		mposname=make_file_name(".mpos");
@@ -246,6 +245,7 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 	}
 	if(mposname) free(mposname);
 	OutputHeader(stdout,*si_mode,lt);
+	if(genv_out) OutputQTLHead(qptr); /* June 3, 1998 EWD */
 	if(analysis&IBD_ANALYSIS) {
 		/* Allocate memory for affected only analyses */
 		if(n_markers) {
@@ -426,7 +426,7 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 			if(!(*si_mode)) {
 				reprune_segs();
 				flag&=~7;
-			}
+			} else flag&=~6;
 			/* Quantitative analysis */
 			if(tlocus) {
 				TL_Birth_Death(peel_work,*si_mode);
@@ -465,6 +465,7 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 			if(sample_freq[0] && lp>=sample_from[0] && !(lp%sample_freq[0])) {
 				for(i=0;i<n_tloci;i++) if(tlocus[i].locus.flag) calc_var_locus(-1-i);
 				OutputSample(fptr,lp);
+				if(genv_out) OutputQTLvect(qptr,lp); /* EWD */
 				if(Freqfile) OutputFreq(ffreq,lp);
 				k=1;
 			}
@@ -472,10 +473,6 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 				if(!k) for(i=0;i<n_tloci;i++) if(tlocus[i].locus.flag) calc_var_locus(-1-i);
 				OutputSample(stdout,lp);
 			}
-			if(dump_freq && lp && !(lp%dump_freq)) {
-				write_dump(lp,lp1,old_pos,flag,lt);
-				dumped=1;
-			} else dumped=0;
 			if(ntl_linked && fpos && lp>=sample_from[0]) {
 				for(k=0;k<=n_links;k++) ntl_linked[k]=0;
 				for(k1=0;k1<n_tloci;k1++) if(tlocus[k1].locus.flag && !(tlocus[k1].locus.flag&TL_LINKED)) ntl_linked[0]++;
@@ -518,6 +515,10 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 				(void)fflush(fpos);
 			}
 		}
+		if(dump_freq && lp && !(lp%dump_freq)) {
+			write_dump(lp,lp1,n_ibd,old_pos,flag,analysis,lt);
+			dumped=1;
+		} else dumped=0;
 		if(fmpos && lp>=sample_from[0]) {
 			if(lp==1) {
 				for(k=k1=0;k1<n_markers;k1++) {
@@ -542,6 +543,7 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 			(void)fflush(fmpos);
 		}
 	}
+	if(!dumped) write_dump(lp-1,lp1,n_ibd,old_pos,flag,analysis,lt);
 	if(analysis&ESTIMATE_IBD) {
 		switch((*ibd_mode)&IBD_MODE_MASK) {
 		 case DEFAULT_IBD_MODE:
@@ -558,8 +560,11 @@ void SampleLoop(struct peel_mem *peel_work,int *si_mode,int read_dump_flag,int a
 			break;
 		}
 	}
-	if(!dumped && !(analysis&IBD_ANALYSIS)) write_dump(lp-1,lp1,old_pos,flag,lt);
 	if(fptr) (void)fclose(fptr);
+	if(genv_out){ /* June 3, 1998 EWD */
+		(void)fclose(qptr);
+		free(qname);
+	}
 	if(fpos) (void)fclose(fpos);
 	if(ffreq) (void)fclose(ffreq);
 	if(polygenic_flag && Polyfile) {
