@@ -40,7 +40,7 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 	n_all=marker[locus].n_all1[comp];
 	n_bits=num_bits(n_all);
 	n_idx=1<<(n_bits+n_bits);
-	mask=(1<<n_bits)-1;
+	mask=(LK_ONE<<n_bits)-LK_ONE;
 	k=n_all*n_all;
 	id_set=work->s2;
 	qval=id_set+k;
@@ -215,11 +215,11 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 		cm[X_MAT]=req_set[X_MAT][locus][kid];
 		cm[X_PAT]=req_set[X_PAT][locus][kid];
 		for(i=0;i<n_all;i++) tmp_idx[i]=0;
-		for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+		for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 			a=a_set[kid][i];
 			if(a) {
 				if(a&cm[X_PAT]) a|=cm[X_PAT];
-				if(i1&cm[X_MAT]) {
+				if(a1&cm[X_MAT]) {
 					b=cm[X_MAT];
 					j=0;
 					while(b) {
@@ -278,7 +278,7 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 				while(a) {
 					if(a&1) {
 						p1=pval[(j<<n_bits)|i];
-						b=(1<<i)|(1<<j);
+						b=(LK_ONE<<i)|(LK_ONE<<j);
 						for(k1=0;k1<nmc;k1++) {
 							k=id_set1[k1];
 							l=id_set2[k1];
@@ -309,7 +309,7 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 				while(a) {
 					if(a&1) {
 						p1=pval[(j<<n_bits)|i];
-						b=(1<<i)|(1<<j);
+						b=(LK_ONE<<i)|(LK_ONE<<j);
 						for(k1=0;k1<nmc;k1++) {
 							k=id_set1[k1];
 							l=id_set2[k1];
@@ -339,7 +339,7 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 				while(a) {
 					if(a&1) {
 						p1=pval[(j<<n_bits)|i];
-						b=(1<<i)|(1<<j);
+						b=(LK_ONE<<i)|(LK_ONE<<j);
 						for(k1=0;k1<nmc;k1++) {
 							k=id_set1[k1];
 							l=id_set2[k1];
@@ -398,9 +398,9 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 				lump=id_array[kid].lumped;
 				l=lump[X_PAT]<<n_bits;
 				j1=1<<n_bits;
-				for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-					id_set1[i]=(cm[X_PAT]&j)?l:i1;
-					id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+				for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+					id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+					id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 				}
 				for(n=0;n<fsp;n++) {
 					i1=id_set1[t_fset->pat_gene[X_MAT]];
@@ -502,9 +502,9 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 				tpp2=id_array[kid].tpp[X_MAT];
 				l=lump[X_PAT]<<n_bits;
 				j1=1<<n_bits;
-				for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-					id_set1[i]=(cm[X_PAT]&j)?l:i1;
-					id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+				for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+					id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+					id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 				}
 				for(n=0;n<fsp;n++) {
 					i1=id_set1[t_fset->pat_gene[X_MAT]];
@@ -596,9 +596,9 @@ double loki_simple_peelop(const struct Simple_Element *element,const int locus,c
 			lump=id_array[pivot].lumped;
 			l=lump[X_PAT]<<n_bits;
 			j1=1<<n_bits;
-			for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-				id_set1[i]=(cm[X_PAT]&j)?l:i1;
-				id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+			for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+				id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+				id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 			}
 			for(n=0;n<fsp;n++) {
 				i1=id_set1[t_fset->pat_gene[X_MAT]];
@@ -736,7 +736,7 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 	n_all=marker[locus].n_all1[comp];
 	n_bits=num_bits(n_all);
 	n_idx=1<<(n_bits+n_bits);
-	mask=(1<<n_bits)-1;
+	mask=(LK_ONE<<n_bits)-LK_ONE;
 	k=n_all*n_all;
 	id_set=work->s2;
 	qval=id_set+k;
@@ -902,11 +902,11 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 		cm[X_MAT]=req_set[X_MAT][locus][kid];
 		cm[X_PAT]=req_set[X_PAT][locus][kid];
 		for(i=0;i<n_all;i++) tmp_idx[i]=0;
-		for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+		for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 			a=a_set[kid][i];
 			if(a) {
 				if(a&cm[X_PAT]) a|=cm[X_PAT];
-				if(i1&cm[X_MAT]) {
+				if(a1&cm[X_MAT]) {
 					b=cm[X_MAT];
 					j=0;
 					while(b) {
@@ -965,7 +965,7 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 				while(a) {
 					if(a&1) {
 						p1=pval[(j<<n_bits)|i];
-						b=(1<<i)|(1<<j);
+						b=(LK_ONE<<i)|(LK_ONE<<j);
 						for(k1=0;k1<nmc;k1++) {
 							k=id_set1[k1];
 							l=id_set2[k1];
@@ -996,7 +996,7 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 				while(a) {
 					if(a&1) {
 						p1=pval[(j<<n_bits)|i];
-						b=(1<<i)|(1<<j);
+						b=(LK_ONE<<i)|(LK_ONE<<j);
 						for(k1=0;k1<nmc;k1++) {
 							k=id_set1[k1];
 							l=id_set2[k1];
@@ -1022,7 +1022,7 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 		while(a1) {
 			if(a1&1) {
 				p1=pval[i];
-				b=(1<<i);
+				b=(LK_ONE<<i);
 				for(k1=0;k1<nmc;k1++) {
 					k=id_set1[k1];
 					l=id_set2[k1];
@@ -1076,9 +1076,9 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 				lump=id_array[kid].lumped;
 				l=lump[X_PAT]<<n_bits;
 				j1=1<<n_bits;
-				for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-					id_set1[i]=(cm[X_PAT]&j)?l:i1;
-					id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+				for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+					id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+					id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 				}
 				for(n=0;n<fsp;n++) {
 					i1=id_set1[t_fset->pat_gene[X_MAT]];
@@ -1180,9 +1180,9 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 				tpp2=id_array[kid].tpp[X_MAT];
 				l=lump[X_PAT]<<n_bits;
 				j1=1<<n_bits;
-				for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-					id_set1[i]=(cm[X_PAT]&j)?l:i1;
-					id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+				for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+					id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+					id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 				}
 				for(n=0;n<fsp;n++) {
 					i1=id_set1[t_fset->pat_gene[X_MAT]];
@@ -1274,9 +1274,9 @@ double loki_simple_peelop_x(const struct Simple_Element *element,const int locus
 			lump=id_array[pivot].lumped;
 			l=lump[X_PAT]<<n_bits;
 			j1=1<<n_bits;
-			for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-				id_set1[i]=(cm[X_PAT]&j)?l:i1;
-				id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+			for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+				id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+				id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 			}
 			for(n=0;n<fsp;n++) {
 				i1=id_set1[t_fset->pat_gene[X_MAT]];
