@@ -24,7 +24,7 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 	int n_all,n_idx,comp,n_bits,*peel_pt2,*id_set1,*id_set2;
 	double *tp,p1,p2,z,z1,prob=0.0,pp[4],*tmp,*tpp1,*tpp2;
 	double *qval,*pval,*mval,*id_set;
-	lk_ulong a,b,cm[2],*cm1[2],cmp,cmm,*t_cm,*t_all1,*t_all2;
+	lk_ulong a,a1,b,b1,cm[2],*cm1[2],cmp,cmm,*t_cm,*t_all1,*t_all2;
 	lk_ulong *tt_all,*peel_pt1;
 	struct fset *peel_fs,*t_fset;
 
@@ -82,7 +82,7 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 		for(m=0;m<n_off;m++) {
 			kid=off[m]-1;
 			cmm=req_set[X_PAT][locus][kid];
-			if(id_array[kid].flag&SAMPLED_MAT) cm1[X_PAT][m]=1<<(id_array[kid].allele[X_PAT]-1);
+			if(id_array[kid].flag&SAMPLED_MAT) cm1[X_PAT][m]=LK_ONE<<(id_array[kid].allele[X_PAT]-1);
 			else cm1[X_PAT][m]=id_array[kid].temp[X_PAT];
 			if(cmm&cm1[X_PAT][m]) cm1[X_PAT][m]|=cmm;
 			off_flag[m]=1;
@@ -113,7 +113,7 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 		for(m=0;m<n_off;m++) {
 			kid=off[m]-1;
  			cmm=req_set[X_MAT][locus][kid];
-			if(id_array[kid].flag&SAMPLED_MAT) cm1[X_MAT][m]=1<<(id_array[kid].allele[X_MAT]-1);
+			if(id_array[kid].flag&SAMPLED_MAT) cm1[X_MAT][m]=LK_ONE<<(id_array[kid].allele[X_MAT]-1);
 			else cm1[X_MAT][m]=id_array[kid].temp[X_MAT];
 			if(cmm&cm1[X_MAT][m]) cm1[X_MAT][m]|=cmm;
 			off_flag[m]=1;
@@ -139,11 +139,11 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 			cm[X_MAT]=req_set[X_MAT][locus][kid];
 			cm[X_PAT]=req_set[X_PAT][locus][kid];
 			for(i=0;i<n_all;i++) t_all1[i]=0;
-			for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 				a=a_set[kid][i];
 				if(a) {
 					if(a&cm[X_PAT]) a|=cm[X_PAT];
-					if(i1&cm[X_MAT]) {
+					if(a1&cm[X_MAT]) {
 						b=cm[X_MAT];
 						j=0;
 						while(b) {
@@ -178,14 +178,14 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 		} else {
 			i=id_array[ids].allele[X_MAT]-1;
 			j=id_array[ids].allele[X_PAT]-1;
-			cmp=(1<<i)|(1<<j);
+			cmp=(LK_ONE<<i)|(LK_ONE<<j);
 			for(k=0;k<n_all;k++) {
 				b=a_set[idd][k];
 				l=0;
 				l1=k;
 				while(b) {
 					if(b&1) {
-						cmm=(1<<k)|(1<<l);
+						cmm=(LK_ONE<<k)|(LK_ONE<<l);
 						t_cm=cm1[X_MAT];
 						for(m=0;m<no;m++) if(!(*(t_cm++)&cmm)) break;
 						if(m==no) {
@@ -216,7 +216,7 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 			l1=i;
 			while(a) {
 				if(a&1) {
-					cmp=(1<<i)|(1<<j);
+					cmp=(LK_ONE<<i)|(LK_ONE<<j);
 					t_cm=cm1[X_PAT];
 					for(m=0;m<no1;m++) if(!(*(t_cm++)&cmp)) break;
 					if(m==no1) {
@@ -244,7 +244,7 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 			l1=k;
 			while(b) {
 				if(b&1) {
-				cmm=(1<<k)|(1<<l);
+				cmm=(LK_ONE<<k)|(LK_ONE<<l);
 					t_cm=cm1[X_MAT];
 					for(m=0;m<no;m++) if(!(*(t_cm++)&cmm)) break;
 					if(m==no) {
@@ -260,13 +260,13 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 		}
 		switch(no2) {
 		 case 1:
-			for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 				a=a_set[ids][i];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
 					if(a&1) {
-						cmp=i1|j1;
+						cmp=a1|b1;
 						t_cm=cm1[X_PAT];
 						for(m=0;m<no1;m++) if(!(*(t_cm++)&cmp)) break;
 						if(m==no1) {
@@ -287,19 +287,19 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 			break;
 		 case 2:
 			t_all1=tt_all+n_all;
-			for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 				a=a_set[ids][i];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
 					if(a&1) {
-						cmp=i1|j1;
+						cmp=a1|b1;
 						t_cm=cm1[X_PAT];
 						for(m=0;m<no1;m++) if(!(*(t_cm++)&cmp)) break;
 						if(m==no1) {
@@ -320,18 +320,18 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 			break;
 		 case 0:
-			for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 				a=a_set[ids][i];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
 					if(a&1) {
-						cmp=i1|j1;
+						cmp=a1|b1;
 						t_cm=cm1[X_PAT];
 						for(m=0;m<no1;m++) if(!(*(t_cm++)&cmp)) break;
 						if(m==no1) {
@@ -348,18 +348,18 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 			break;
 		 default:
-			for(i1=1,i=0;i<n_all;i++,i1<<=1) {
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1) {
 				a=a_set[ids][i];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
 					if(a&1) {
-						cmp=i1|j1;
+						cmp=a1|b1;
 						t_cm=cm1[X_PAT];
 						for(m=0;m<no1;m++) if(!(*(t_cm++)&cmp)) break;
 						if(m==no1) {
@@ -382,7 +382,7 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 		}
@@ -420,9 +420,9 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 				lump=id_array[kid].lumped;
 				l=lump[X_PAT]<<n_bits;
 				j1=1<<n_bits;
-				for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-					id_set1[i]=(cm[X_PAT]&j)?l:i1;
-					id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+				for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+					id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+					id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 				}
 				for(n=0;n<fsp;n++) {
 					i1=id_set1[t_fset->pat_gene[X_MAT]];
@@ -495,9 +495,9 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 				lump=id_array[kid].lumped;
 				l=lump[X_PAT]<<n_bits;
 				j1=1<<n_bits;
-				for(i=i1=0,j=1;i<n_all;i++,j<<=1,i1+=j1) {
-					id_set1[i]=(cm[X_PAT]&j)?l:i1;
-					id_set2[i]=(cm[X_MAT]&j)?lump[X_MAT]:i;
+				for(i=i1=0,a1=LK_ONE;i<n_all;i++,a1<<=1,i1+=j1) {
+					id_set1[i]=(cm[X_PAT]&a1)?l:i1;
+					id_set2[i]=(cm[X_MAT]&a1)?lump[X_MAT]:i;
 				}
 				tpp1=id_array[kid].tpp[X_PAT];
 				tpp2=id_array[kid].tpp[X_MAT];
@@ -615,10 +615,10 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 		j=id_array[ids].allele[X_PAT]-1;
 		k=id_array[idd].allele[X_MAT]-1;
 		l=id_array[idd].allele[X_PAT]-1;
-		if((1<<i)&cm[X_PAT]) i=id_array[kid].lumped[X_PAT];
-		if((1<<j)&cm[X_PAT]) j=id_array[kid].lumped[X_PAT];
-		if((1<<k)&cm[X_MAT]) k=id_array[kid].lumped[X_MAT];
-		if((1<<l)&cm[X_MAT]) l=id_array[kid].lumped[X_MAT];
+		if((LK_ONE<<i)&cm[X_PAT]) i=id_array[kid].lumped[X_PAT];
+		if((LK_ONE<<j)&cm[X_PAT]) j=id_array[kid].lumped[X_PAT];
+		if((LK_ONE<<k)&cm[X_MAT]) k=id_array[kid].lumped[X_MAT];
+		if((LK_ONE<<l)&cm[X_MAT]) l=id_array[kid].lumped[X_MAT];
 		i1=i<<n_bits;
 		j1=j<<n_bits;
 		p1=(pp[X_MM_PM]=tp[X_MM_PM]*qval[i1|k]);
@@ -675,7 +675,6 @@ double loki_simple_sample(const struct Simple_Element *element,const int locus,p
 		(void) printf("Returning from %s() with %g\n",FUNC_NAME,prob);
 	}
 #endif
-	
 	return prob;
 }
 
