@@ -32,15 +32,38 @@
 #include "gen_pen.h"
 #include "meiosis_scan.h"
 
-static int *nnfd_st,*par_st,*gpfam_st,*fam_st;
+static int *nnfd_st,*par_st,*gpfam_st,*fam_st,mem_list_size,mem_list_ptr;
 static int *nnfd_list,*par_list,*fam_list,*gpfam_list,*temp_list;
 static struct nuc_family *families;
+static void **mem_list;
 
 double mscan_prob[]={MSCAN_INDIVIDUAL,
 	  MSCAN_HS_FAMILY,
 	  MSCAN_GP_FAMILY,
 	  MSCAN_FS_FAMILY};
 
+#ifdef FUNC_NAME
+#undef FUNC_NAME
+#endif
+#define FUNC_NAME "malloc_and_remember"
+static void *malloc_and_remember(size_t size)
+{
+	void *p;
+	
+	if(mem_list_ptr==mem_list_size) {
+		if(mem_list_size) {
+			mem_list_size*=1.5;
+			if(!(mem_list=realloc(mem_list,sizeof(void *)*mem_list_size))) ABT_FUNC(MMsg);
+		} else {
+			mem_list_size=16;
+			if(!(mem_list=malloc(sizeof(void *)*mem_list_size))) ABT_FUNC(MMsg);
+		}
+	}
+	p=malloc(size);
+	if(p) mem_list[mem_list_ptr++]=p;
+	return p;
+}
+
 #ifdef FUNC_NAME
 #undef FUNC_NAME
 #endif
@@ -50,11 +73,11 @@ static void init_families(void)
 	int j,i1,cs,comp,nnfd,npar,nfam,ngpfam;
 	int k,k1,k2,nk,kid,ids,idd,*kid_list;
 	
-	if(!(nnfd_st=malloc(sizeof(int)*(n_comp+1)*4))) ABT_FUNC(MMsg);
+	if(!(nnfd_st=malloc_and_remember(sizeof(int)*(n_comp+1)*4))) ABT_FUNC(MMsg);
 	par_st=nnfd_st+n_comp+1;
 	fam_st=par_st+n_comp+1;
 	gpfam_st=fam_st+n_comp+1;
-	if(!(nnfd_list=malloc(sizeof(int)*3*ped_size))) ABT_FUNC(MMsg);
+	if(!(nnfd_list=malloc_and_remember(sizeof(int)*3*ped_size))) ABT_FUNC(MMsg);
 	par_list=nnfd_list+ped_size;
 	temp_list=par_list+ped_size;
 	for(j=0;j<ped_size;j++) id_array[j].flag=0;
@@ -89,8 +112,8 @@ static void init_families(void)
 		for(comp=0;comp<=n_comp;comp++) fam_st[comp]=gpfam_st[comp]=0;
 		return;
 	}
-	if(!(families=malloc(sizeof(struct nuc_family)*nfam))) ABT_FUNC(MMsg);
-	if(!(fam_list=malloc(sizeof(int)*(nfam+ngpfam+nk)))) ABT_FUNC(MMsg);
+	if(!(families=malloc_and_remember(sizeof(struct nuc_family)*nfam))) ABT_FUNC(MMsg);
+	if(!(fam_list=malloc_and_remember(sizeof(int)*(nfam+ngpfam+nk)))) ABT_FUNC(MMsg);
 	gpfam_list=fam_list+nfam;
 	kid_list=gpfam_list+ngpfam;
 	for(nk=nfam=ngpfam=j=comp=0;comp<n_comp;comp++) {
@@ -123,6 +146,11 @@ static void init_families(void)
 	gpfam_st[comp]=ngpfam;
 }
   
+static void free_meiosis_scan(void)
+{
+	meiosis_scan(-1,0);
+}
+
 #ifdef FUNC_NAME
 #undef FUNC_NAME
 #endif
@@ -136,18 +164,13 @@ void meiosis_scan(int link,int si)
 	static double *recom[2],*pp[4],*pen[4],*lks[4],**lk_store;
 	static int *loci,*loc_fg,***seg_list,ctr;
 	static struct Locus **loci1;
-	void **mptr[]={(void **)pp,(void **)&loci,(void **)&nnfd_list,
-		  (void **)&nnfd_st,(void **)&families,(void **)&fam_list,
-		  (void **)&seg_list,(void **)&loci1,0};
-	void **p;
 
 	if(link<0) { /* Free space and prevent reuse */
-		i=0;
-		while((p=mptr[i++])) {
-			if(*p) {
-				free(*p);
-				*p=0;
-			}
+		if(mem_list) {
+			for(i=0;i<mem_list_ptr;i++) if(mem_list[i]) free(mem_list[i]);
+			free(mem_list);
+			mem_list=0;
+			mem_list_size=mem_list_ptr=0;
 		}
 		return;
 	}
@@ -159,7 +182,7 @@ void meiosis_scan(int link,int si)
 			if(k>j) j=k;
 		}
 		j+=max_tloci;
-		if(!(pp[0]=malloc(sizeof(double)*(14*j-1)))) ABT_FUNC(MMsg);
+		if(!(pp[0]=malloc_and_remember(sizeof(double)*(14*j-1)))) ABT_FUNC(MMsg);
 		pen[0]=pp[0]+4*j;
 		lks[0]=pen[0]+4*j;
 		recom[0]=lks[0]+4*j;
@@ -169,18 +192,18 @@ void meiosis_scan(int link,int si)
 			pen[i]=pen[i-1]+j;
 			lks[i]=lks[i-1]+j;
 		}
-		if(!(seg_list=malloc(sizeof(void *)*j*2))) ABT_FUNC(MMsg);
+		if(!(seg_list=malloc_and_remember(sizeof(void *)*j*2))) ABT_FUNC(MMsg);
 		lk_store=(void *)(seg_list+j);
-		if(!(loci=malloc(sizeof(int)*j*2))) ABT_FUNC(MMsg);
+		if(!(loci=malloc_and_remember(sizeof(int)*j*2))) ABT_FUNC(MMsg);
 		loc_fg=loci+j;
-		if(!(loci1=malloc(sizeof(void *)*j))) ABT_FUNC(MMsg);
+		if(!(loci1=malloc_and_remember(sizeof(void *)*j))) ABT_FUNC(MMsg);
 		init_families();
 		/* Normalize mscan type probabilities */
 		for(z1=0.0,update_type=0;update_type<4;update_type++) z1+=mscan_prob[update_type];
 		if(z1<=0.0) ABT_FUNC("Invalid probabilities set for mscan_prob[]\n");
 		z1=1.0/z1;
 		for(update_type=0;update_type<4;update_type++) mscan_prob[update_type]*=z1;
-		
+		atexit(free_meiosis_scan);
 	}
 	z=safe_ranf();
 	for(z1=0.0,update_type=0;update_type<4;update_type++) {
