@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <sys/wait.h>
+#include <limits.h>
 
 #include "utils.h"
 #include "libhdr.h"
@@ -392,17 +393,21 @@ void ReadBinFiles(char **lfile,int eflag)
 		RemBlock=AddRemem(tmp,RemBlock);
 		if(fread(tmp,1,(size_t)k,fptr)!=(size_t)k) BFE(fname,EMsg[3]);
 		for(k1=j=0;j<ped_size;j++) {
-			id_recode.recode[j].string=tmp+k1;
-			while(k1<k && tmp[k1]!='\n') k1++;
-			if(k1>=k) BFE(fname,EMsg[0]);
+			if(tmp[k1]!='\n') {
+				id_recode.recode[j].string=tmp+k1;
+				while(k1<k && tmp[k1]!='\n') k1++;
+				if(k1>=k) BFE(fname,EMsg[0]);
+			} else id_recode.recode[j].string=0;
 			tmp[k1++]='\0';
 		}
 	} else {
 		id_recode.flag=ST_INTEGER;
 		for(j=0;j<ped_size;j++)	{
 			if(!(tmp=fget_line(fptr))) BFE(fname,EMsg[3]);
-			id_recode.recode[j].value=(int)strtol(tmp,&tmp1,16);
-			if(*tmp1!='\n') BFE(fname,EMsg[2]);
+			if(*tmp!='\n') {
+				id_recode.recode[j].value=(int)strtol(tmp,&tmp1,16);
+				if(*tmp1!='\n') BFE(fname,EMsg[2]);
+			} else id_recode.recode[j].value=INT_MAX;
 		}
 	}
 	if(n_links) {
@@ -894,14 +899,26 @@ void ReadBinFiles(char **lfile,int eflag)
 									if(!eflag && tmp1[-1]=='\n') BFE(fname,"File appears to be for Fenris.  Do not use prep -e to prepare datafiles for Loki");
 									BFE(fname,EMsg[2]);
 								}
+#ifdef USE_LONGLONG
+								tl[k1]=strtoll(tmp1,&tmp,16);
+#else
 								tl[k1]=strtol(tmp1,&tmp,16);
+#endif
 								tmp1=tmp;
 							}
 							if(v2[3]) {	
 								if(*tmp1++!=',') BFE(fname,EMsg[2]);
+#ifdef USE_LONGLONG
+								req_set[0][i][k+j]=strtoll(tmp1,&tmp,16);
+#else
 								req_set[0][i][k+j]=strtol(tmp1,&tmp,16);
+#endif
 								if(*tmp++!=',') BFE(fname,EMsg[2]);
+#ifdef USE_LONGLONG
+								req_set[1][i][k+j]=strtoll(tmp,&tmp1,16);
+#else
 								req_set[1][i][k+j]=strtol(tmp,&tmp1,16);
+#endif
 							}
 							tl+=v2[2];
 						}
