@@ -340,10 +340,10 @@ void init_sample_effects(void)
 	}
 	mat[i]=k3;
 	/* Get factorization order */
-	min_deg(n_lev,mat,order1,0);
+	min_deg(n_lev,mat,order1,0,0);
 	for(i=0;i<n_lev;i++) order[order1[i]]=i+1;
 	free(order1);
-	min_deg(0,0,0,0);
+	min_deg(0,0,0,0,0);
 }
 
 void free_sample_effects(void)
