@@ -131,7 +131,7 @@ static void print_code_rec_all(FILE *fptr,int ch,lk_ulong a,int locus)
 {
 	int l,k;
 	
-	if(a&(1<<ch)) {
+	if(a&(LK_ONE<<ch)) {
 		l=k=0;
 		while(a) {
 			if(a&1) {
@@ -157,11 +157,11 @@ static void print_alls(FILE *fptr,const int i,const int locus,const int n_all,co
 	if(!(b=calloc(n_all,sizeof(lk_ulong)))) ABT_FUNC(MMsg);
 	j=ped_recode1[i-1]-1-id;
 	if(linktype==LINK_Y || (linktype==LINK_X && id_array[i-1].sex==1)) fg=1;
-	m=(1<<n_all)-1;
+	m=(LK_ONE<<n_all)-LK_ONE;
 	for(k1=k=0;k<n_all;k++)	{
 		a=all_set[k][j];
 		if(a==m) k1++;
-		for(l=0;l<n_all;l++) if(a&(1<<l)) b[l]|=1<<k;
+		for(l=0;l<n_all;l++) if(a&(LK_ONE<<l)) b[l]|=LK_ONE<<k;
 	}
 	if(k1==n_all) {
 		if(fg) (void)fputs("(*)",fptr);
@@ -267,9 +267,9 @@ static void PrintFamily(const int fam,const int locus,const int n_all,const int
 #define FUNC_NAME "DoFamily"
 static int DoFamily(const int fam,const int n_all,const int locus,const int linktype)
 {
-	int i,j,k,l,i1,j1,k1,l1,m,ids,idd,nc=0,fg,kid,change=0;
+	int i,j,k,k1,l,m,ids,idd,nc=0,fg,kid,change=0;
 	int nkids,*tmp,nmc;
-	lk_ulong a,b,c,cm,cm1,*t_all1,*t_all2;
+	lk_ulong a,a1,b,b1,c,cm,cm1,*t_all1,*t_all2;
 	static int *kids,*m_set1,*m_set2,max_kids,max_k,max_mc;
 	static lk_ulong *ccm,*ccm1,*tt_all;
 
@@ -337,7 +337,7 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 			for(k=0;k<n_all;k++) {
 				if((b=t_all1[k])) {
 					a|=b;
-					c|=1<<k;
+					c|=LK_ONE<<k;
 				}
 			}
 			ccm[nkids]=a;
@@ -348,13 +348,13 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 	}
 	for(i=0;i<n_all;i++) temp_set[i]=id_set[0][i]=id_set[1][i]=0;
 	if(linktype==LINK_AUTO)	{
-		for(k1=1,nmc=k=0;k<n_all;k++,k1<<=1) {
+		for(a=LK_ONE,nmc=k=0;k<n_all;k++,a<<=1) {
 			b=all_set[k][idd];
 			l=0;
-			l1=1;
+			a1=LK_ONE;
 			while(b) {
-				if((b&1)&&(l<=k || !(all_set[l][idd]&k1))) {
-					cm=k1|l1;
+				if((b&1)&&(l<=k || !(all_set[l][idd]&a))) {
+					cm=a|a1;
 					for(m=0;m<nkids;m++) if(!(cm&ccm1[m])) break;
 					if(m==nkids) {
 						m_set1[nmc]=k;
@@ -363,18 +363,18 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 				}
 				b>>=1;
 				l++;
-				l1<<=1;
+				a1<<=1;
 			}
 		}
 		switch(nkids) {
 		 case 1:
-			for(i1=1,i=0;i<n_all;i++,i1<<=1)	{
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1)	{
 				a=all_set[i][ids];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
-					if((a&1)&&(j<=i || !(all_set[j][ids]&i1))) {
-						cm=i1|j1;
+					if((a&1)&&(j<=i || !(all_set[j][ids]&a1))) {
+						cm=a1|b1;
 						for(m=0;m<nkids;m++) if(!(cm&ccm[m])) break;
 						if(m==nkids) for(k1=0;k1<nmc;k1++) {
 							k=m_set1[k1];
@@ -382,29 +382,29 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 							if((tt_all[k]&cm)||(tt_all[l]&cm)) {
 								temp_set[k]|=cm;
 								temp_set[l]|=cm;
-								id_set[0][i]|=j1;
-								id_set[0][j]|=i1;
-								id_set[1][k]|=1<<l;
-								id_set[1][l]|=1<<k;
+								id_set[0][i]|=b1;
+								id_set[0][j]|=a1;
+								id_set[1][k]|=LK_ONE<<l;
+								id_set[1][l]|=LK_ONE<<k;
 								nc++;
 							}
 						}
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 			break;
 		 case 2:
 			t_all1=tt_all+n_all;
-			for(i1=1,i=0;i<n_all;i++,i1<<=1)	{
+			for(a1=1,i=0;i<n_all;i++,a1<<=1)	{
 				a=all_set[i][ids];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
-					if((a&1)&&(j<=i || !(all_set[j][ids]&i1))) {
-						cm=i1|j1;
+					if((a&1)&&(j<=i || !(all_set[j][ids]&a1))) {
+						cm=a1|b1;
 						for(m=0;m<nkids;m++) if(!(cm&ccm[m])) break;
 						if(m==nkids) for(k1=0;k1<nmc;k1++) {
 							k=m_set1[k1];
@@ -412,30 +412,30 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 							if(((tt_all[k]&cm)||(tt_all[l]&cm))&&((t_all1[k]&cm)||(t_all1[l]&cm))) {
 								temp_set[k]|=cm;
 								temp_set[l]|=cm;
-								id_set[0][i]|=j1;
-								id_set[0][j]|=i1;
-								id_set[1][k]|=1<<l;
-								id_set[1][l]|=1<<k;
+								id_set[0][i]|=b1;
+								id_set[0][j]|=a1;
+								id_set[1][k]|=LK_ONE<<l;
+								id_set[1][l]|=LK_ONE<<k;
 								nc++;
 							}
 						}
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 			break;
 		 case 3:
 			t_all1=tt_all+n_all;
 			t_all2=t_all1+n_all;
-			for(i1=1,i=0;i<n_all;i++,i1<<=1)	{
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1)	{
 				a=all_set[i][ids];
 				j=0;
-				j1=1;
+				b1=1;
 				while(a) {
-					if((a&1)&&(j<=i || !(all_set[j][ids]&i1))) {
-						cm=i1|j1;
+					if((a&1)&&(j<=i || !(all_set[j][ids]&a1))) {
+						cm=a1|b1;
 						for(m=0;m<nkids;m++) if(!(cm&ccm[m])) break;
 						if(m==nkids) for(k1=0;k1<nmc;k1++) {
 							k=m_set1[k1];
@@ -443,28 +443,28 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 							if(((tt_all[k]&cm)||(tt_all[l]&cm))&&((t_all1[k]&cm)||(t_all1[l]&cm))&&((t_all2[k]&cm)||(t_all2[l]&cm))) {
 								temp_set[k]|=cm;
 								temp_set[l]|=cm;
-								id_set[0][i]|=j1;
-								id_set[0][j]|=i1;
-								id_set[1][k]|=1<<l;
-								id_set[1][l]|=1<<k;
+								id_set[0][i]|=b1;
+								id_set[0][j]|=a1;
+								id_set[1][k]|=LK_ONE<<l;
+								id_set[1][l]|=LK_ONE<<k;
 								nc++;
 							}
 						}
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 			break;
 		 default:
-			for(i1=1,i=0;i<n_all;i++,i1<<=1)	{
+			for(a1=LK_ONE,i=0;i<n_all;i++,a1<<=1)	{
 				a=all_set[i][ids];
 				j=0;
-				j1=1;
+				b1=LK_ONE;
 				while(a) {
-					if((a&1)&&(j<=i || !(all_set[j][ids]&i1))) {
-						cm=i1|j1;
+					if((a&1)&&(j<=i || !(all_set[j][ids]&a1))) {
+						cm=a1|b1;
 						for(m=0;m<nkids;m++) if(!(cm&ccm[m])) break;
 						if(m==nkids) for(k1=0;k1<nmc;k1++) {
 							k=m_set1[k1];
@@ -474,17 +474,17 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 							if(m==nkids) {
 								temp_set[k]|=cm;
 								temp_set[l]|=cm;
-								id_set[0][i]|=j1;
-								id_set[0][j]|=i1;
-								id_set[1][k]|=1<<l;
-								id_set[1][l]|=1<<k;
+								id_set[0][i]|=b1;
+								id_set[0][j]|=a1;
+								id_set[1][k]|=LK_ONE<<l;
+								id_set[1][l]|=LK_ONE<<k;
 								nc++;
 							}
 						}
 					}
 					a>>=1;
 					j++;
-					j1<<=1;
+					b1<<=1;
 				}
 			}
 		}
@@ -492,9 +492,9 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 		for(i=0;i<n_all;i++) if(all_set[i][ids]) {
 			for(k=0;k<n_all;k++)	{
 				b=all_set[k][idd];
-				if(b) for(l=0;l<n_all;l++) if(b&(1<<l)) {
-					if(l>k && (all_set[l][idd]&(1<<k))) continue;
-					cm=(1<<i);
+				if(b) for(l=0;l<n_all;l++) if(b&(LK_ONE<<l)) {
+					if(l>k && (all_set[l][idd]&(LK_ONE<<k))) continue;
+					cm=(LK_ONE<<i);
 					fg=0;
 					for(m=0;m<family[fam].nkids;m++)	{
 						kid=family[fam].kids[m];
@@ -509,18 +509,18 @@ static int DoFamily(const int fam,const int n_all,const int locus,const int link
 					if(fg) continue;
 					temp_set[k]|=cm;
 					temp_set[l]|=cm;
-					id_set[0][i]|=1;
-					id_set[1][k]|=1<<l;
-					id_set[1][l]|=1<<k;
+					id_set[0][i]|=LK_ONE;
+					id_set[1][k]|=LK_ONE<<l;
+					id_set[1][l]|=LK_ONE<<k;
 					nc++;
 				}
 			}
 		}
 	} else if(linktype==LINK_Y) {
 		a=all_set[0][ids];
-		for(i=0;i<n_all;i++) if(a&(1<<i)) {
+		for(i=0;i<n_all;i++) if(a&(LK_ONE<<i)) {
 			fg=0;
-			cm=1<<i;
+			cm=LK_ONE<<i;
 			for(m=0;m<family[fam].nkids;m++)	{
 				kid=family[fam].kids[m];
 				if((id_array[kid].flag&IS_PRUNED) || id_array[kid].sex==2) continue;
@@ -1070,10 +1070,12 @@ static int *assemble_matrix(int size,int *g_perm,int *trans,int k2,const int lin
 #undef FUNC_NAME
 #endif
 #define FUNC_NAME "min_degree"
-static int *min_degree(int *g_perm,int size,const int linktype)
+static int *min_degree(int *g_perm,int size,const int linktype,int locus)
 {
-	int x,i1,k1,k2,*trans;
-	int *mat,*order;
+	int x,i1,k1,k2,*trans,best_fg=0;
+	int *mat,*order,*order_bk,*t;
+	double *wt,cost,best=-1.0,z;
+	struct pair_wt *wt1=0;
 	
 	k2=INT_MAX;
 	k1=INT_MIN;
@@ -1090,13 +1092,57 @@ static int *min_degree(int *g_perm,int size,const int linktype)
 		i1=g_perm[x]+k2;
 		trans[i1]=x;
 	}
-	mat=assemble_matrix(size,g_perm,trans,k2,linktype);
-	free(trans);
-	if(!mat) ABT_FUNC("Internal error - assemble_matrix() returned a zero pointer\n");
-	if(!(order=malloc(sizeof(int)*size))) ABT_FUNC(MMsg);
-	min_deg(size,mat,order,0);
-	free(mat);
-	return order;
+	if(1) {
+		if(!(wt=malloc(sizeof(double)*size))) ABT_FUNC(MMsg);
+		if(locus<n_markers) {
+			if(!(wt1=malloc(sizeof(struct pair_wt)*size))) ABT_FUNC(MMsg);
+			for(x=0;x<size;x++) wt1[x].pair_node=-1;
+			for(x=0;x<size;x++) {
+				i1=g_perm[x];
+				if(i1>0) {
+					wt[x]=log((double)id_array[i1-1].nhaps[X_MAT]);
+					k1=trans[k2-i1];
+					if(k1>=0) {
+						wt1[x].pair_node=k1;
+						wt1[k1].pair_node=x;
+						wt1[x].wt=wt1[k1].wt=log((double)id_array[i1-1].ngens);
+					}
+				} else wt[x]=log((double)id_array[-1-i1].nhaps[X_PAT]);
+			}
+		} else {
+			z=log(2.0);
+			for(x=0;x<size;x++) wt[x]=z;
+		}
+		if(!(order=malloc(sizeof(int)*size))) ABT_FUNC(MMsg);
+		if(!(order_bk=malloc(sizeof(int)*size))) ABT_FUNC(MMsg);
+		for(x=0;x<4;x++) {
+			mat=assemble_matrix(size,g_perm,trans,k2,linktype);
+			if(!mat) ABT_FUNC("Internal error - assemble_matrix() returned a zero pointer\n");
+			greedy(size,mat,order,0,wt,wt1,x,&cost);
+			if(best<0.0 || cost<best) {
+				best=cost;
+				best_fg=x;
+				t=order;
+				order=order_bk;
+				order_bk=t;
+			}
+/*			printf("%d %g\n",x,cost); */
+			free(mat);
+		}
+/*		printf("best_fg=%d, score=%g\n",best_fg,best); */
+		free(order);
+		free(wt);
+		if(wt1) free(wt1);
+	} else {
+		mat=assemble_matrix(size,g_perm,trans,k2,linktype);
+		if(!mat) ABT_FUNC("Internal error - assemble_matrix() returned a zero pointer\n");
+		free(trans);
+		if(!(order=malloc(sizeof(int)*size))) ABT_FUNC(MMsg);
+		min_deg(size,mat,order,0,2);
+		free(mat);
+		order_bk=order;
+	}
+	return order_bk;
 }
 
 #ifdef FUNC_NAME
@@ -1146,7 +1192,7 @@ static int find_sequence(const int n_fam,const int *famlist,const int n_all,stru
 	}
 	if(locus==n_markers) mark=traitlocus;
 	else mark=markers+locus;
-	order=min_degree(g_perm,n_genes,linktype);
+	order=min_degree(g_perm,n_genes,linktype,locus);
 	for(j=old_rfuncs;j<n_rfuncs;j++) free(r_func[j].id_list);
 	n_rfuncs=old_rfuncs;
 	set_flags(n_fam,famlist,comp,linktype);
@@ -1180,18 +1226,35 @@ static void fill_rf(int pivot,int n_all,int n_bits,struct Peelseq_Head *pp)
 	piv1=ped_recode1[pivot-1]-1-id;
 	for(k1=k2=0;k2<n_all;k2++) {
 		a=all_set[k2][piv1];
-		if(a) for(k3=0;k3<n_all;k3++) if(a&(1<<k3)) k1++;
+		if(a) for(k3=0;k3<n_all;k3++) if(a&(LK_ONE<<k3)) k1++;
 	}
 	r_func[k].n_terms=k1;
 	if(!(r_func[k].index=malloc(k1*sizeof(lk_ulong)))) ABT_FUNC(MMsg);
 	for(k1=k2=0;k2<n_all;k2++) {
 		a=all_set[k2][piv1];
-		if(a) for(k3=0;k3<n_all;k3++) if(a&(1<<k3)) {
+		if(a) for(k3=0;k3<n_all;k3++) if(a&(LK_ONE<<k3)) {
 			r_func[k].index[k1++]=(lk_ulong)((k3<<n_bits)|k2);
 		}
 	}
 }
 
+static char *get_errfile(const char *s,int fg)
+{
+	char *fname1=0,*fname2;
+	
+	if(ErrorDir) {
+		if((fname1=malloc(strlen(ErrorDir)+strlen(s)+(fg?7:6))))
+		  (void)sprintf(fname1,"%s/%s.er%s",ErrorDir,s,fg?"r1":"r");
+	} else {
+		if((fname2=malloc(strlen(s)+(fg?6:5)))) {
+			(void)sprintf(fname2,"%s.er%s",s,fg?"r1":"r");
+			fname1=add_file_dir(fname2);
+			free(fname2);
+		}
+	}
+	return fname1;
+}
+
 #ifdef FUNC_NAME
 #undef FUNC_NAME
 #endif
@@ -1248,7 +1311,7 @@ static int logical_peel(const int n_fam,int *famlist,const int n_all,const int l
 			if(!a) continue;
 			k1++;
 			b|=a;
-			if(a&(1<<k)) k4++;
+			if(a&(LK_ONE<<k)) k4++;
 			while(a) {
 				if(a&1) k3++;
 				a>>=1;
@@ -1300,7 +1363,7 @@ static int logical_peel(const int n_fam,int *famlist,const int n_all,const int l
 			a=req_set[X_MAT][j];
 			k1=k2=0;
 			for(k=0;k<n_all;k++) if(all_set[k][j]) {
-				b=1<<k;
+				b=LK_ONE<<k;
 				if(a&b) b=a;
 				if(all_set[k][idd]) k1++;
 				c=req_set[2][idd];
@@ -1813,7 +1876,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 		req_set[0]=all_set[i-1]+comp_size[comp];
 		req_set[1]=req_set[0]+comp_size[comp];
 		req_set[2]=req_set[1]+comp_size[comp];
-		mask=(1<<n_all1)-1;
+		mask=(LK_ONE<<n_all1)-LK_ONE;
 		for(j=0;j<comp_size[comp];j++) {
 			i=perm[j+id];
 			ids=id_array[i].sire;
@@ -1844,8 +1907,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 						if(id_array[i].sex==2) (void)fputs(" is female and has genetic data - deleting\n",stderr);
 						else (void)fputs(" has 2 different haplotypes - deleting\n",stderr);
 						if(!fname1)	{
-							if(!(fname1=malloc(strlen(marker_name)+6))) ABT_FUNC(MMsg);
-							(void)sprintf(fname1,"%s.err1",marker_name);
+							if(!(fname1=get_errfile(marker_name,1))) ABT_FUNC(MMsg);
 							ferr=fopen(fname1,"w");
 						}
 						if(ferr)	{
@@ -1864,8 +1926,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 						print_orig_id(stderr,i+1,0);
 						(void)fputs(" is male with 2 different haplotypes - deleting\n",stderr);
 						if(!fname1)	{
-							if(!(fname1=malloc(strlen(marker_name)+6))) ABT_FUNC(MMsg);
-							(void)sprintf(fname1,"%s.err1",marker_name);
+							if(!(fname1=get_errfile(marker_name,1))) ABT_FUNC(MMsg);
 							ferr=fopen(fname1,"w");
 						}
 						if(ferr) {
@@ -1882,10 +1943,10 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 				}
 				if(ch[1]) {
 					for(k=0;k<n_all1;k++) all_set[k][j]=0;
-					all_set[ch[0]-1][j]=1<<(ch[1]-1);
-					all_set[ch[1]-1][j]=1<<(ch[0]-1);
+					all_set[ch[0]-1][j]=LK_ONE<<(ch[1]-1);
+					all_set[ch[1]-1][j]=LK_ONE<<(ch[0]-1);
 				} else if(ch[0]) {
-					m1=1<<(ch[0]-1);
+					m1=LK_ONE<<(ch[0]-1);
 					if(linktype && id_array[i].sex==1) {
 						for(k1=0;k1<n_all1;k1++) all_set[k1][j]=0;
 						if(linktype==LINK_X) all_set[ch[0]-1][j]=1;
@@ -1975,13 +2036,13 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 					} else ch[0]=ch[1]=0;
 					if(linktype==LINK_Y || (linktype==LINK_X && sex==1)) {
 						if(ch[0]) {
-							req_set[linktype==LINK_Y?X_PAT:X_MAT][j]|=1<<(ch[0]-1);
+							req_set[linktype==LINK_Y?X_PAT:X_MAT][j]|=LK_ONE<<(ch[0]-1);
 							continue;
 						}
 					} else {
 						if(ch[0]) {
 							for(k=0;k<2;k++) if(ch[k])	{
-								m1=1<<(ch[k]-1);
+								m1=LK_ONE<<(ch[k]-1);
 								if(all_set[ch[k]-1][j]) req_set[X_MAT][j]|=m1;
 								for(k1=0;k1<n_all1;k1++) if(all_set[k1][j]&m1) req_set[X_PAT][j]|=m1;
 							}
@@ -2005,7 +2066,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 							b=all_set[k1][j];
 							if(b)	{
 								if(id_array[kid].sex==2 && sex==2) req_set[X_PAT][j]|=(a&b);
-								m1=1<<k1;
+								m1=LK_ONE<<k1;
 								if(a&m1) req_set[X_MAT][j]|=m1;
 							}
 						}
@@ -2015,7 +2076,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 							b=all_set[k1][j];
 							if(b) {
 								req_set[X_PAT][j]|=(a&b);
-								m1=1<<k1;
+								m1=LK_ONE<<k1;
 								if(a&m1) req_set[X_MAT][j]|=m1;
 							}
 						}
@@ -2033,7 +2094,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 					a=all_set[k1][j];
 					if(!a) continue;
 					if(linktype!=LINK_Y)	{
-						m1=1<<k1;
+						m1=LK_ONE<<k1;
 						req_set[X_MAT][j]^=m1;
 					}
 					b|=a;
@@ -2076,7 +2137,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 						for(b=0,k1=0;k1<n_all1;k1++) {
 							a=all_set[k1][j];
 							if(!a) continue;
-							m1=1<<k1;
+							m1=LK_ONE<<k1;
 							if(req_set[X_MAT][j]&m1) {
 								b|=a;
 								all_set[k1][j]=0;
@@ -2085,7 +2146,7 @@ static int Check_Locus(const int locus,FILE *fptr,char *fname,int component,cons
 						all_set[km-1][j]=b;
 					}
 					if(kp) {
-						m1=1<<(kp-1);
+						m1=LK_ONE<<(kp-1);
 						for(k1=0;k1<n_all1;k1++) {
 							a=(req_set[X_PAT][j]&all_set[k1][j]);
 							if(a) {
@@ -2207,13 +2268,7 @@ int Genotype_Elimination(int check_flag,char *lfile,int error_check)
 				markers[locus].element->n_levels=n_all_old;
 				if(fam<0) ABT_FUNC(AbMsg);
 				(void)fputs("Genotype inconsistency - searching for errors\n",stderr);
-				if(ErrorDir) {
-					if(!(fname1=malloc(strlen(ErrorDir)+strlen(marker_name)+6))) ABT_FUNC(MMsg);
-					(void)sprintf(fname1,"%s/%s.err",ErrorDir,marker_name);
-				} else {
-					if(!(fname1=malloc(strlen(marker_name)+5))) ABT_FUNC(MMsg);
-					(void)sprintf(fname1,"%s.err",marker_name);
-				}
+				if(!(fname1=get_errfile(marker_name,0))) ABT_FUNC(MMsg);
 				if((ferr=fopen(fname1,"r"))) {
 					k1=0;
 					(void)fprintf(stderr,"Reading 'bad' individuals from %s\n",fname1);
@@ -2520,6 +2575,6 @@ int Genotype_Elimination(int check_flag,char *lfile,int error_check)
 		free(blank);
 	}
 	DoFamily(-1,0,0,0);
-	min_deg(0,0,0,0);
+	min_deg(0,0,0,0,0);
 	return locus<(n_markers+(traitlocus?1:0))?1:0;
 }
