@@ -48,7 +48,7 @@ static char *default_rsformat="\n";
 static size_t StringPos;
 static struct miss_var_tag *miss_var;
 static struct DataBlock *DataBlock;
-static struct bin_node *node_strings,*node_ints,*node_ped_int,*node_ped_str;
+static struct bin_node *node_strings,*node_ints,*node_ped_int,*node_fam_int,*node_ped_str,*node_fam_str;
 static int num_nodes;
 struct label_data **ped_recode,**family_recode,***factor_recode;
 int *ped_recode1;
@@ -176,6 +176,7 @@ static struct bin_node *alloc_node(const void *s,int type)
 	struct bin_node *node;
 	struct label_data *data;
 	size_t i;
+	char *p,*p1;
 	
 	if(!(node=malloc(sizeof(struct bin_node)))) ABT_FUNC(MMsg);
 	node->left=node->right=0;
@@ -186,14 +187,21 @@ static struct bin_node *alloc_node(const void *s,int type)
 	data->index=num_nodes++;
 	switch(type) {
 	 case STRING:
-		i=strlen((char *)s)+1;
+		p1=(char *)s;
+		i=strlen(p1)+1;
 		if(i>(S_BLOCK_SIZE-StringPos)) {
 			if(!(StringData=malloc(S_BLOCK_SIZE))) ABT_FUNC(MMsg);
 			RemBlock=AddRemem(StringData,RemBlock);
 			StringPos=0;
 		}
-		(void)strcpy(StringData+StringPos,(char *)s);
-		data->data.string=StringData+StringPos;
+		p=StringData+StringPos;
+		data->data.string=p;
+		if(!syst_var[IGNORE_CASE]) (void)strcpy(p,p1);
+		else {
+			p1=(char *)s;
+			while(*p1) *p++=toupper((int)(*p1++));
+			*p=0;
+		}
 		StringPos+=i;
 		break;
 	 case INTEGER:
@@ -236,7 +244,8 @@ static struct label_data *find_snode(struct bin_node *node,const char *s)
 	struct label_data *data;
 	
 	data=node->data;
-	i=strcasecmp(s,data->data.string);
+	if(!syst_var[IGNORE_CASE]) i=strcmp(s,data->data.string);
+	else i=strcasecmp(s,data->data.string);
 	if(i<0) {
 		if(node->left) data=find_snode(node->left,s);
 		else data=0;
@@ -258,7 +267,10 @@ static struct bin_node *insert_node(struct bin_node *node,const void *s,struct l
 	
 	data=node->data;
 	if(type==INTEGER) i=*((long *)s)-data->data.value;
-	else 	i=strcasecmp((char *)s,data->data.string);
+	else {
+		if(!syst_var[IGNORE_CASE]) i=strcmp(s,data->data.string);
+		else i=strcasecmp((char *)s,data->data.string);
+	}
 	if(i<0) {
 		if(node->left) node->left=insert_node(node->left,s,node1,balanced,type);
 		else {
@@ -314,11 +326,11 @@ struct label_data *find_node(const void *s,int type,int flag)
 	struct label_data *data=0;
 
 	if(type==INTEGER) {
-		if(flag) node=node_ints;
+		if(flag) node=node_fam_int;
 		else node=node_ped_int;
 		if(node) data=find_inode(node,*((long *)s));
 	} else {
-		if(flag) node=node_strings;
+		if(flag) node=node_fam_str;
 		else node=node_ped_str;
 		if(node) data=find_snode(node,(char *)s);
 	}
@@ -331,7 +343,9 @@ void free_nodes(void)
 	if(node_strings) free_bin_tree(node_strings,free);
 	if(node_ped_int) free_bin_tree(node_ped_int,free);
 	if(node_ped_str) free_bin_tree(node_ped_str,free);
-	node_ints=node_strings=node_ped_int=node_ped_str=0;
+	if(node_fam_int) free_bin_tree(node_fam_int,free);
+	if(node_fam_str) free_bin_tree(node_fam_str,free);
+	node_ints=node_strings=node_ped_int=node_ped_str=node_fam_int=node_fam_str=0;
 }
 
 static void set_missing(int col,int ncol,int rec,struct DataBlock *db)
@@ -381,7 +395,7 @@ static int check_miss(char *string,struct express *miss,int elem_type)
 			ep.arg.value=strtol(miss->arg.string,&p,10);
 			if(!*p) ep.type=ST_INTEGER;
 		}
-	}
+	} 
 	if(!ep.type) {
 		ep.type=miss->type;
 		ep.arg=miss->arg;
@@ -442,7 +456,7 @@ static int handle_string(char *string,struct InFile *infile,struct var_element *
 		}
 	}
 	if(!miss) {
-		if(elem->type&(ST_ID|ST_SIRE|ST_DAM|ST_FAMILY|ST_FACTOR|ST_HAPLO)) {
+		if(elem->type&(ST_ID|ST_SIRE|ST_DAM|ST_FAMILY|ST_FACTOR|ST_HAPLO|ST_MARKER)) {
 			if(elem->type&ST_MARKER) {
 				/* Genotype data - split into fields based on gsformat */
 				p1=string;
@@ -558,11 +572,16 @@ static int handle_string(char *string,struct InFile *infile,struct var_element *
 						  print_scan_err("[%s:%d] %s(): Line %d column %d - Malformed integer %s\n",__FILE__,__LINE__,FUNC_NAME,lineno,col+1,string);
 						miss=1;
 					} else {
-						if(elem->type&(ST_ID|ST_SIRE|ST_DAM|ST_RANDOM))	{
+						if(elem->type&(ST_ID|ST_SIRE|ST_DAM))	{
 							if(!node_ped_int) {
 								node_ped_int=alloc_node(&value,INTEGER);
 								node=node_ped_int->data;
 							} else node_ped_int=insert_node(node_ped_int,&value,&node,&i,INTEGER);
+						} else if(elem->type&(ST_FAMILY)) {
+							if(!node_fam_int) {
+								node_fam_int=alloc_node(&value,INTEGER);
+								node=node_fam_int->data;
+							} else node_fam_int=insert_node(node_fam_int,&value,&node,&i,INTEGER);
 						} else {
 							if(!node_ints) {
 								node_ints=alloc_node(&value,INTEGER);
@@ -572,11 +591,16 @@ static int handle_string(char *string,struct InFile *infile,struct var_element *
 						DataBlock->records[DataBlock->record_ptr*ncol+col].node=node;
 					}
 				} else {
-					if(elem->type&(ST_ID|ST_SIRE|ST_DAM|ST_RANDOM))	{
+					if(elem->type&(ST_ID|ST_SIRE|ST_DAM))	{
 						if(!node_ped_str) {
 							node_ped_str=alloc_node(string,STRING);
 							node=node_ped_str->data;
 						} else node_ped_str=insert_node(node_ped_str,string,&node,&i,STRING);
+					} else if(elem->type&(ST_FAMILY)) {
+						if(!node_fam_str) {
+							node_fam_str=alloc_node(string,STRING);
+							node=node_fam_str->data;
+						} else node_fam_str=insert_node(node_fam_str,string,&node,&i,STRING);
 					} else {
 						if(!node_strings) {
 							node_strings=alloc_node(string,STRING);
@@ -634,7 +658,7 @@ void ReadData(char *lfile)
 {
 	int i,j,ncol,num_records,col,realcol,eor,miss;
 	struct InFile *infile;
-	char *buffer,ch,*string,*string_start,oldchar,*rs,*fs,*gs,*gs1;
+	char *buffer,ch,*string,*string_start,oldchar,*rs,*fs,*gs,*gs1,*tname;
 	int buffersize,fs_reg=0,skip,skip_this;
 	FILE *fptr,*flog;
 	size_t l,string_leng;
@@ -651,8 +675,10 @@ void ReadData(char *lfile)
 	if(!(StringData=malloc(S_BLOCK_SIZE))) ABT_FUNC(MMsg);
 	RemBlock=AddRemem(StringData,RemBlock);
 	StringPos=0;
-	if(lfile) flog=fopen(lfile,"a");
-	else flog=0;
+	if(lfile && (tname=add_file_dir(lfile))) {
+		flog=fopen(tname,"a");
+		free(tname);
+	} else flog=0;
 	if(flog) i=fputs("\n******************** Reading in data ********************\n\n",flog);
 	while(infile) {
 		fs_reg=0;
@@ -839,8 +865,10 @@ void ReadData(char *lfile)
 						}
 						string_start=string-1;
 						elem=infile->element[realcol];
-						if(elem && elem->type&ST_MARKER) i=1;
-						else i=0;
+						if(elem && elem->type&ST_MARKER) {
+							if(gs && *gs && (!isspace((int)*gs) || gs[1])) i=0;
+							else i=1;
+						} else i=0;
 						if(!eor)	{
 							while((ch= *(string++))) {
 								if(rs) if(ch==rs[0]) {
@@ -947,6 +975,17 @@ void ReadData(char *lfile)
 	if(scan_error_n || !num_nodes) return;
 	if(flog && flog!=stdout) (void)fclose(flog);
 	if(!(recode_table=calloc((size_t)num_nodes,sizeof(struct recode_table_tag)))) ABT_FUNC(MMsg);
+	for(i=j=0;i<n_markers;i++) {
+		if(!(markers[i].element->type&ST_NOT_REALLY_REQUIRED)) {
+			memcpy(markers+j,markers+i,sizeof(struct Marker));
+			j++;
+		}
+	}
+	n_markers=j;
+	if(!j) {
+		free(markers);
+		markers=0;
+	}
 	setup_pedigree(num_nodes,recode_table,lfile);
 	if(scan_error_n) return;
 	for(i=0;i<num_nodes;i++) recode_table[i].index=0;
@@ -973,6 +1012,7 @@ void ReadData(char *lfile)
 		sd=sex_def;
 	}
 	prune_pedigree(lfile);
+	check_sex2();
 	count_components(lfile);
 /*	count_relationships(); */
 	if(OutputFile) Output_Data();
