@@ -165,6 +165,7 @@ void count_components(char *LogFile)
 {
 	int i,j,k,*group,*equiv,*code,*perm,*nbits,ids,idd;
 	FILE *flog;
+	char *tname;
 	
 	n_comp=k=0;
 	if(!(group=calloc((size_t)(3*ped_size),sizeof(int)))) ABT_FUNC(MMsg);
@@ -246,8 +247,10 @@ void count_components(char *LogFile)
 		}
 		if(verbose&1) (void)printf("No. components = %d\n",k);
 		if((verbose&2)) {
-			if(LogFile) flog=fopen(LogFile,"a");
-			else flog=0;
+  			if(LogFile && (tname=add_file_dir(LogFile))) {
+				flog=fopen(tname,"a");
+				free(tname);
+			} else flog=0;
 			if(flog)	{
 				(void)fputs("\n****************** Counting components ******************\n\n",flog);
 				(void)fprintf(flog,"     No. components = %d\n",k);
@@ -277,6 +280,7 @@ void setup_pedigree(int num_nodes,struct recode_table_tag *recode_table,char *Lo
 	struct DataBlock *db;
 	struct var_element *elem;
 	FILE *flog;
+	char *tname;
 	
 	if(verbose&1) (void)printf("Initializing and recoding pedigree\n");
 	infile=Infiles;
@@ -479,8 +483,10 @@ void setup_pedigree(int num_nodes,struct recode_table_tag *recode_table,char *Lo
 		}
 	}
 	if(verbose&2) {
-		if(LogFile) flog=fopen(LogFile,"a");
-		else flog=0;
+		if(LogFile && (tname=add_file_dir(LogFile))) {
+			flog=fopen(tname,"a");
+			free(tname);
+		} else flog=0;
 		if(flog) {
 			(void)fprintf(flog,"\n******************* Recoding pedigree *******************\n\n");
 			(void)fprintf(flog,"     No. individuals found = %d\n",ped_size-n_dummy);
@@ -499,6 +505,7 @@ void handle_groups(char *LogFile)
 {
 	int i,j,k,k1,sire,dam,er=0,*ct,*ct1;
 	FILE *flog;
+	char *tname;
 	
  	for(i=0;i<ped_size;i++) id_array[i].group=0;
 	if(!group_elem) return;
@@ -553,8 +560,10 @@ void handle_groups(char *LogFile)
 	n_genetic_groups=k;
 	for(i=0;i<ped_size;i++) if(id_array[i].group) id_array[i].group=ct1[id_array[i].group-1];
 	if(verbose&1) (void)printf("No. genetic groups = %d\n",k);
-	if((verbose&2) && LogFile) flog=fopen(LogFile,"a");
-	else flog=0;
+	if((verbose&2) && LogFile && (tname=add_file_dir(LogFile))) {
+		flog=fopen(tname,"a");
+		free(tname);
+	} else flog=0;
 	if(flog)	{
 		(void)fprintf(flog,"\n********************* Genetic groups ********************\n\n");
 		(void)fputs("         No. genetics groups       No. Records\n",flog);
@@ -580,6 +589,7 @@ void prune_pedigree(char *LogFile)
 {
 	int i,i1,j,k,tj,*n_kids=0,sire,dam,*perm;
 	FILE *flog;
+	char *tname;
 	
 	if((verbose&1) && syst_var[PRUNE_OPTION]) (void)printf("Pruning pedigree\n");
 	/* Flag which individuals have trait and marker data */
@@ -679,8 +689,10 @@ void prune_pedigree(char *LogFile)
 	while(j);
 	if(tj) {
 		if(verbose&1) (void)printf("No. pruned = %d\n",tj);
-		if((verbose&2) && LogFile) flog=fopen(LogFile,"a");
-		else flog=0;
+		if((verbose&2) && LogFile && (tname=add_file_dir(LogFile))) {
+			flog=fopen(tname,"a");
+			free(tname);
+		} else flog=0;
 		if(flog) {
 			(void)fprintf(flog,"\n******************** Pruning pedigree *******************\n\n");
 			(void)fprintf(flog,"     No. individuals pruned = %d, IDs follow:\n",tj);
@@ -784,6 +796,28 @@ void check_sex(void)
 	for(i=0;i<ped_size;i++) id_array[i].sex&=3;
 }
 
+#ifdef FUNC_NAME
+#undef FUNC_NAME
+#endif
+#define FUNC_NAME "check_sex2"
+void check_sex2(void)
+{
+	int i,sex_reqd=0,er=0;
+	struct Link *linkp;
+	
+	linkp=links;
+	while(linkp && !sex_reqd) {
+		if(linkp->type!=LINK_AUTO) sex_reqd=1;
+		linkp=linkp->next;
+	}
+	if(sex_reqd) for(i=0;i<ped_size;i++) if(ped_recode1[i] && !id_array[i].sex) {
+		print_orig_id(stderr,i+1,0);
+		fputs(" has no sex information (required for sex linked loci)\n",stderr);
+		er=1;
+	}
+	if(er) ABT_FUNC("Pedigree errors - aborting\n");
+}
+
 void inbr_rep(FILE *fptr,int i,int ids,int idd,char *inbr[])
 {
 	static int fg;
@@ -852,10 +886,13 @@ void check_inbreeding(char *LogFile)
 	static char *inbr[]={"full sib","half sib","offspring parent",
 		"grandchild grandparent","niece/nephew aunt/uncle","first cousin","descendent ancestor"};
 	FILE *flog;
-
+	char *tname;
+	
 	for(i=0;i<7;i++) cc[i]=0;
-	if(LogFile) flog=fopen(LogFile,"a");
-	else flog=0;
+	if(LogFile && (tname=add_file_dir(LogFile))) {
+		flog=fopen(tname,"a");
+		free(tname);
+	} else flog=0;
 	for(i=0;i<n_families;i++) {
 		ids=family[i].sire;
 		idd=family[i].dam;
@@ -921,9 +958,12 @@ void count_loops(char *LogFile)
 {
 	int i,j,k1,k2,k3,k4,pivot,kid,fam,ids,idd,*famlist,nf,*compflag;
 	FILE *flog;
+	char *tname;
 	
-	if(LogFile) flog=fopen(LogFile,"a");
-	else flog=0;
+	if(LogFile && (tname=add_file_dir(LogFile))) {
+		flog=fopen(tname,"a");
+		free(tname);
+	} else flog=0;
 	if(flog) (void)fputs("\n********************* Counting loops *********************\n",flog);
 	for(i=0;i<ped_size;i++) id_array[i].order=0;
 	if(!(compflag=malloc(sizeof(int)*(n_comp+n_families)))) ABT_FUNC(MMsg);
